
''' Generated by OTF2 Template Engine '''

import ctypes

from .Config import conf, StrParam
from .ErrorCodes import ErrorCode, HandleErrorCode
from .GeneralDefinitions import *
from .AttributeValue import AttributeValue


class AttributeList(ctypes.Structure):
    pass

def AttributeList_New():
    c_New = conf.lib.OTF2_AttributeList_New
    c_New.argtypes = []
    c_New.restype = ctypes.POINTER(AttributeList)
    # NOTE: Do not errcheck here. This function returns a value, not an error code
    return c_New()

def AttributeList_Delete(attributeList):
    c_Delete = conf.lib.OTF2_AttributeList_Delete
    c_Delete.argtypes = [ ctypes.POINTER(AttributeList) ]
    c_Delete.restype = ErrorCode
    c_Delete.errcheck = HandleErrorCode
    c_Delete(attributeList)

def AttributeList_AddAttribute(attributeList, attribute, type, attributeValue):
    c_AddAttribute = conf.lib.OTF2_AttributeList_AddAttribute
    c_AddAttribute.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, Type, AttributeValue._arg_type ]
    c_AddAttribute.restype = ErrorCode
    c_AddAttribute.errcheck = HandleErrorCode
    c_AddAttribute(attributeList, attribute, type, attributeValue._arg_value)

def AttributeList_AddUint8(attributeList, attribute, uint8Value):
    c_AddUint8 = conf.lib.OTF2_AttributeList_AddUint8
    c_AddUint8.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.c_uint8 ]
    c_AddUint8.restype = ErrorCode
    c_AddUint8.errcheck = HandleErrorCode
    c_AddUint8(attributeList, attribute, uint8Value)

def AttributeList_AddUint16(attributeList, attribute, uint16Value):
    c_AddUint16 = conf.lib.OTF2_AttributeList_AddUint16
    c_AddUint16.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.c_uint16 ]
    c_AddUint16.restype = ErrorCode
    c_AddUint16.errcheck = HandleErrorCode
    c_AddUint16(attributeList, attribute, uint16Value)

def AttributeList_AddUint32(attributeList, attribute, uint32Value):
    c_AddUint32 = conf.lib.OTF2_AttributeList_AddUint32
    c_AddUint32.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.c_uint32 ]
    c_AddUint32.restype = ErrorCode
    c_AddUint32.errcheck = HandleErrorCode
    c_AddUint32(attributeList, attribute, uint32Value)

def AttributeList_AddUint64(attributeList, attribute, uint64Value):
    c_AddUint64 = conf.lib.OTF2_AttributeList_AddUint64
    c_AddUint64.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.c_uint64 ]
    c_AddUint64.restype = ErrorCode
    c_AddUint64.errcheck = HandleErrorCode
    c_AddUint64(attributeList, attribute, uint64Value)

def AttributeList_AddInt8(attributeList, attribute, int8Value):
    c_AddInt8 = conf.lib.OTF2_AttributeList_AddInt8
    c_AddInt8.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.c_int8 ]
    c_AddInt8.restype = ErrorCode
    c_AddInt8.errcheck = HandleErrorCode
    c_AddInt8(attributeList, attribute, int8Value)

def AttributeList_AddInt16(attributeList, attribute, int16Value):
    c_AddInt16 = conf.lib.OTF2_AttributeList_AddInt16
    c_AddInt16.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.c_int16 ]
    c_AddInt16.restype = ErrorCode
    c_AddInt16.errcheck = HandleErrorCode
    c_AddInt16(attributeList, attribute, int16Value)

def AttributeList_AddInt32(attributeList, attribute, int32Value):
    c_AddInt32 = conf.lib.OTF2_AttributeList_AddInt32
    c_AddInt32.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.c_int32 ]
    c_AddInt32.restype = ErrorCode
    c_AddInt32.errcheck = HandleErrorCode
    c_AddInt32(attributeList, attribute, int32Value)

def AttributeList_AddInt64(attributeList, attribute, int64Value):
    c_AddInt64 = conf.lib.OTF2_AttributeList_AddInt64
    c_AddInt64.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.c_int64 ]
    c_AddInt64.restype = ErrorCode
    c_AddInt64.errcheck = HandleErrorCode
    c_AddInt64(attributeList, attribute, int64Value)

def AttributeList_AddFloat(attributeList, attribute, float32Value):
    c_AddFloat = conf.lib.OTF2_AttributeList_AddFloat
    c_AddFloat.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.c_float ]
    c_AddFloat.restype = ErrorCode
    c_AddFloat.errcheck = HandleErrorCode
    c_AddFloat(attributeList, attribute, float32Value)

def AttributeList_AddDouble(attributeList, attribute, float64Value):
    c_AddDouble = conf.lib.OTF2_AttributeList_AddDouble
    c_AddDouble.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.c_double ]
    c_AddDouble.restype = ErrorCode
    c_AddDouble.errcheck = HandleErrorCode
    c_AddDouble(attributeList, attribute, float64Value)

def AttributeList_AddString(attributeList, attribute, stringRef):
    c_AddString = conf.lib.OTF2_AttributeList_AddString
    c_AddString.argtypes = [ctypes.POINTER(AttributeList), AttributeRef, StringRef]
    c_AddString.restype = ErrorCode
    c_AddString.errcheck = HandleErrorCode
    c_AddString(attributeList, attribute, stringRef)

def AttributeList_AddStringRef(attributeList, attribute, stringRef):
    c_AddStringRef = conf.lib.OTF2_AttributeList_AddStringRef
    c_AddStringRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, StringRef ]
    c_AddStringRef.restype = ErrorCode
    c_AddStringRef.errcheck = HandleErrorCode
    c_AddStringRef(attributeList, attribute, stringRef)

def AttributeList_AddAttributeRef(attributeList, attribute, attributeRef):
    c_AddAttributeRef = conf.lib.OTF2_AttributeList_AddAttributeRef
    c_AddAttributeRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, AttributeRef ]
    c_AddAttributeRef.restype = ErrorCode
    c_AddAttributeRef.errcheck = HandleErrorCode
    c_AddAttributeRef(attributeList, attribute, attributeRef)

def AttributeList_AddLocationRef(attributeList, attribute, locationRef):
    c_AddLocationRef = conf.lib.OTF2_AttributeList_AddLocationRef
    c_AddLocationRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, LocationRef ]
    c_AddLocationRef.restype = ErrorCode
    c_AddLocationRef.errcheck = HandleErrorCode
    c_AddLocationRef(attributeList, attribute, locationRef)

def AttributeList_AddRegionRef(attributeList, attribute, regionRef):
    c_AddRegionRef = conf.lib.OTF2_AttributeList_AddRegionRef
    c_AddRegionRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, RegionRef ]
    c_AddRegionRef.restype = ErrorCode
    c_AddRegionRef.errcheck = HandleErrorCode
    c_AddRegionRef(attributeList, attribute, regionRef)

def AttributeList_AddGroupRef(attributeList, attribute, groupRef):
    c_AddGroupRef = conf.lib.OTF2_AttributeList_AddGroupRef
    c_AddGroupRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, GroupRef ]
    c_AddGroupRef.restype = ErrorCode
    c_AddGroupRef.errcheck = HandleErrorCode
    c_AddGroupRef(attributeList, attribute, groupRef)

def AttributeList_AddMetricRef(attributeList, attribute, metricRef):
    c_AddMetricRef = conf.lib.OTF2_AttributeList_AddMetricRef
    c_AddMetricRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, MetricRef ]
    c_AddMetricRef.restype = ErrorCode
    c_AddMetricRef.errcheck = HandleErrorCode
    c_AddMetricRef(attributeList, attribute, metricRef)

def AttributeList_AddCommRef(attributeList, attribute, commRef):
    c_AddCommRef = conf.lib.OTF2_AttributeList_AddCommRef
    c_AddCommRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, CommRef ]
    c_AddCommRef.restype = ErrorCode
    c_AddCommRef.errcheck = HandleErrorCode
    c_AddCommRef(attributeList, attribute, commRef)

def AttributeList_AddParameterRef(attributeList, attribute, parameterRef):
    c_AddParameterRef = conf.lib.OTF2_AttributeList_AddParameterRef
    c_AddParameterRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ParameterRef ]
    c_AddParameterRef.restype = ErrorCode
    c_AddParameterRef.errcheck = HandleErrorCode
    c_AddParameterRef(attributeList, attribute, parameterRef)

def AttributeList_AddRmaWinRef(attributeList, attribute, rmaWinRef):
    c_AddRmaWinRef = conf.lib.OTF2_AttributeList_AddRmaWinRef
    c_AddRmaWinRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, RmaWinRef ]
    c_AddRmaWinRef.restype = ErrorCode
    c_AddRmaWinRef.errcheck = HandleErrorCode
    c_AddRmaWinRef(attributeList, attribute, rmaWinRef)

def AttributeList_AddSourceCodeLocationRef(attributeList, attribute, sourceCodeLocationRef):
    c_AddSourceCodeLocationRef = conf.lib.OTF2_AttributeList_AddSourceCodeLocationRef
    c_AddSourceCodeLocationRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, SourceCodeLocationRef ]
    c_AddSourceCodeLocationRef.restype = ErrorCode
    c_AddSourceCodeLocationRef.errcheck = HandleErrorCode
    c_AddSourceCodeLocationRef(attributeList, attribute, sourceCodeLocationRef)

def AttributeList_AddCallingContextRef(attributeList, attribute, callingContextRef):
    c_AddCallingContextRef = conf.lib.OTF2_AttributeList_AddCallingContextRef
    c_AddCallingContextRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, CallingContextRef ]
    c_AddCallingContextRef.restype = ErrorCode
    c_AddCallingContextRef.errcheck = HandleErrorCode
    c_AddCallingContextRef(attributeList, attribute, callingContextRef)

def AttributeList_AddInterruptGeneratorRef(attributeList, attribute, interruptGeneratorRef):
    c_AddInterruptGeneratorRef = conf.lib.OTF2_AttributeList_AddInterruptGeneratorRef
    c_AddInterruptGeneratorRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, InterruptGeneratorRef ]
    c_AddInterruptGeneratorRef.restype = ErrorCode
    c_AddInterruptGeneratorRef.errcheck = HandleErrorCode
    c_AddInterruptGeneratorRef(attributeList, attribute, interruptGeneratorRef)

def AttributeList_AddIoFileRef(attributeList, attribute, ioFileRef):
    c_AddIoFileRef = conf.lib.OTF2_AttributeList_AddIoFileRef
    c_AddIoFileRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, IoFileRef ]
    c_AddIoFileRef.restype = ErrorCode
    c_AddIoFileRef.errcheck = HandleErrorCode
    c_AddIoFileRef(attributeList, attribute, ioFileRef)

def AttributeList_AddIoHandleRef(attributeList, attribute, ioHandleRef):
    c_AddIoHandleRef = conf.lib.OTF2_AttributeList_AddIoHandleRef
    c_AddIoHandleRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, IoHandleRef ]
    c_AddIoHandleRef.restype = ErrorCode
    c_AddIoHandleRef.errcheck = HandleErrorCode
    c_AddIoHandleRef(attributeList, attribute, ioHandleRef)

def AttributeList_AddLocationGroupRef(attributeList, attribute, locationGroupRef):
    c_AddLocationGroupRef = conf.lib.OTF2_AttributeList_AddLocationGroupRef
    c_AddLocationGroupRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, LocationGroupRef ]
    c_AddLocationGroupRef.restype = ErrorCode
    c_AddLocationGroupRef.errcheck = HandleErrorCode
    c_AddLocationGroupRef(attributeList, attribute, locationGroupRef)

def AttributeList_RemoveAttribute(attributeList, attribute):
    c_RemoveAttribute = conf.lib.OTF2_AttributeList_RemoveAttribute
    c_RemoveAttribute.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef ]
    c_RemoveAttribute.restype = ErrorCode
    c_RemoveAttribute.errcheck = HandleErrorCode
    c_RemoveAttribute(attributeList, attribute)

def AttributeList_RemoveAllAttributes(attributeList):
    c_RemoveAllAttributes = conf.lib.OTF2_AttributeList_RemoveAllAttributes
    c_RemoveAllAttributes.argtypes = [ ctypes.POINTER(AttributeList) ]
    c_RemoveAllAttributes.restype = ErrorCode
    c_RemoveAllAttributes.errcheck = HandleErrorCode
    c_RemoveAllAttributes(attributeList)

def AttributeList_TestAttributeByID(attributeList, attribute):
    c_TestAttributeByID = conf.lib.OTF2_AttributeList_TestAttributeByID
    c_TestAttributeByID.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef ]
    c_TestAttributeByID.restype = ctypes.c_bool
    return c_TestAttributeByID(attributeList, attribute)

def AttributeList_GetAttributeByID(attributeList, attribute, attributeType = Type(), attributeValue = AttributeValue()):
    c_GetAttributeByID = conf.lib.OTF2_AttributeList_GetAttributeByID
    c_GetAttributeByID.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_GetAttributeByID.restype = ErrorCode
    c_GetAttributeByID.errcheck = HandleErrorCode
    c_GetAttributeByID(attributeList, attribute, ctypes.byref(attributeType), ctypes.byref(attributeValue))
    return (attributeType, attributeValue)

def AttributeList_GetUint8(attributeList, attribute):
    c_GetUint8 = conf.lib.OTF2_AttributeList_GetUint8
    c_GetUint8.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(ctypes.c_uint8) ]
    c_GetUint8.restype = ErrorCode
    c_GetUint8.errcheck = HandleErrorCode
    uint8Value = ctypes.c_uint8()
    c_GetUint8(attributeList, attribute, ctypes.byref(uint8Value))
    return uint8Value.value

def AttributeList_GetUint16(attributeList, attribute):
    c_GetUint16 = conf.lib.OTF2_AttributeList_GetUint16
    c_GetUint16.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(ctypes.c_uint16) ]
    c_GetUint16.restype = ErrorCode
    c_GetUint16.errcheck = HandleErrorCode
    uint16Value = ctypes.c_uint16()
    c_GetUint16(attributeList, attribute, ctypes.byref(uint16Value))
    return uint16Value.value

def AttributeList_GetUint32(attributeList, attribute):
    c_GetUint32 = conf.lib.OTF2_AttributeList_GetUint32
    c_GetUint32.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(ctypes.c_uint32) ]
    c_GetUint32.restype = ErrorCode
    c_GetUint32.errcheck = HandleErrorCode
    uint32Value = ctypes.c_uint32()
    c_GetUint32(attributeList, attribute, ctypes.byref(uint32Value))
    return uint32Value.value

def AttributeList_GetUint64(attributeList, attribute):
    c_GetUint64 = conf.lib.OTF2_AttributeList_GetUint64
    c_GetUint64.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(ctypes.c_uint64) ]
    c_GetUint64.restype = ErrorCode
    c_GetUint64.errcheck = HandleErrorCode
    uint64Value = ctypes.c_uint64()
    c_GetUint64(attributeList, attribute, ctypes.byref(uint64Value))
    return uint64Value.value

def AttributeList_GetInt8(attributeList, attribute):
    c_GetInt8 = conf.lib.OTF2_AttributeList_GetInt8
    c_GetInt8.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(ctypes.c_int8) ]
    c_GetInt8.restype = ErrorCode
    c_GetInt8.errcheck = HandleErrorCode
    int8Value = ctypes.c_int8()
    c_GetInt8(attributeList, attribute, ctypes.byref(int8Value))
    return int8Value.value

def AttributeList_GetInt16(attributeList, attribute):
    c_GetInt16 = conf.lib.OTF2_AttributeList_GetInt16
    c_GetInt16.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(ctypes.c_int16) ]
    c_GetInt16.restype = ErrorCode
    c_GetInt16.errcheck = HandleErrorCode
    int16Value = ctypes.c_int16()
    c_GetInt16(attributeList, attribute, ctypes.byref(int16Value))
    return int16Value.value

def AttributeList_GetInt32(attributeList, attribute):
    c_GetInt32 = conf.lib.OTF2_AttributeList_GetInt32
    c_GetInt32.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(ctypes.c_int32) ]
    c_GetInt32.restype = ErrorCode
    c_GetInt32.errcheck = HandleErrorCode
    int32Value = ctypes.c_int32()
    c_GetInt32(attributeList, attribute, ctypes.byref(int32Value))
    return int32Value.value

def AttributeList_GetInt64(attributeList, attribute):
    c_GetInt64 = conf.lib.OTF2_AttributeList_GetInt64
    c_GetInt64.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(ctypes.c_int64) ]
    c_GetInt64.restype = ErrorCode
    c_GetInt64.errcheck = HandleErrorCode
    int64Value = ctypes.c_int64()
    c_GetInt64(attributeList, attribute, ctypes.byref(int64Value))
    return int64Value.value

def AttributeList_GetFloat(attributeList, attribute):
    c_GetFloat = conf.lib.OTF2_AttributeList_GetFloat
    c_GetFloat.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(ctypes.c_float) ]
    c_GetFloat.restype = ErrorCode
    c_GetFloat.errcheck = HandleErrorCode
    float32Value = ctypes.c_float()
    c_GetFloat(attributeList, attribute, ctypes.byref(float32Value))
    return float32Value.value

def AttributeList_GetDouble(attributeList, attribute):
    c_GetDouble = conf.lib.OTF2_AttributeList_GetDouble
    c_GetDouble.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(ctypes.c_double) ]
    c_GetDouble.restype = ErrorCode
    c_GetDouble.errcheck = HandleErrorCode
    float64Value = ctypes.c_double()
    c_GetDouble(attributeList, attribute, ctypes.byref(float64Value))
    return float64Value.value

def AttributeList_GetString(attributeList, attribute):
    c_GetString = conf.lib.OTF2_AttributeList_GetString
    c_GetString.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(StringRef) ]
    c_GetString.restype = ErrorCode
    c_GetString.errcheck = HandleErrorCode
    stringRef = StringRef()
    c_GetString(attributeList, attribute, ctypes.byref(stringRef))
    return stringRef.value

def AttributeList_GetStringRef(attributeList, attribute):
    c_GetStringRef = conf.lib.OTF2_AttributeList_GetStringRef
    c_GetStringRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(StringRef) ]
    c_GetStringRef.restype = ErrorCode
    c_GetStringRef.errcheck = HandleErrorCode
    stringRef = StringRef()
    c_GetStringRef(attributeList, attribute, ctypes.byref(stringRef))
    return stringRef.value

def AttributeList_GetAttributeRef(attributeList, attribute):
    c_GetAttributeRef = conf.lib.OTF2_AttributeList_GetAttributeRef
    c_GetAttributeRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(AttributeRef) ]
    c_GetAttributeRef.restype = ErrorCode
    c_GetAttributeRef.errcheck = HandleErrorCode
    attributeRef = AttributeRef()
    c_GetAttributeRef(attributeList, attribute, ctypes.byref(attributeRef))
    return attributeRef.value

def AttributeList_GetLocationRef(attributeList, attribute):
    c_GetLocationRef = conf.lib.OTF2_AttributeList_GetLocationRef
    c_GetLocationRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(LocationRef) ]
    c_GetLocationRef.restype = ErrorCode
    c_GetLocationRef.errcheck = HandleErrorCode
    locationRef = LocationRef()
    c_GetLocationRef(attributeList, attribute, ctypes.byref(locationRef))
    return locationRef.value

def AttributeList_GetRegionRef(attributeList, attribute):
    c_GetRegionRef = conf.lib.OTF2_AttributeList_GetRegionRef
    c_GetRegionRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(RegionRef) ]
    c_GetRegionRef.restype = ErrorCode
    c_GetRegionRef.errcheck = HandleErrorCode
    regionRef = RegionRef()
    c_GetRegionRef(attributeList, attribute, ctypes.byref(regionRef))
    return regionRef.value

def AttributeList_GetGroupRef(attributeList, attribute):
    c_GetGroupRef = conf.lib.OTF2_AttributeList_GetGroupRef
    c_GetGroupRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(GroupRef) ]
    c_GetGroupRef.restype = ErrorCode
    c_GetGroupRef.errcheck = HandleErrorCode
    groupRef = GroupRef()
    c_GetGroupRef(attributeList, attribute, ctypes.byref(groupRef))
    return groupRef.value

def AttributeList_GetMetricRef(attributeList, attribute):
    c_GetMetricRef = conf.lib.OTF2_AttributeList_GetMetricRef
    c_GetMetricRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(MetricRef) ]
    c_GetMetricRef.restype = ErrorCode
    c_GetMetricRef.errcheck = HandleErrorCode
    metricRef = MetricRef()
    c_GetMetricRef(attributeList, attribute, ctypes.byref(metricRef))
    return metricRef.value

def AttributeList_GetCommRef(attributeList, attribute):
    c_GetCommRef = conf.lib.OTF2_AttributeList_GetCommRef
    c_GetCommRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(CommRef) ]
    c_GetCommRef.restype = ErrorCode
    c_GetCommRef.errcheck = HandleErrorCode
    commRef = CommRef()
    c_GetCommRef(attributeList, attribute, ctypes.byref(commRef))
    return commRef.value

def AttributeList_GetParameterRef(attributeList, attribute):
    c_GetParameterRef = conf.lib.OTF2_AttributeList_GetParameterRef
    c_GetParameterRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(ParameterRef) ]
    c_GetParameterRef.restype = ErrorCode
    c_GetParameterRef.errcheck = HandleErrorCode
    parameterRef = ParameterRef()
    c_GetParameterRef(attributeList, attribute, ctypes.byref(parameterRef))
    return parameterRef.value

def AttributeList_GetRmaWinRef(attributeList, attribute):
    c_GetRmaWinRef = conf.lib.OTF2_AttributeList_GetRmaWinRef
    c_GetRmaWinRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(RmaWinRef) ]
    c_GetRmaWinRef.restype = ErrorCode
    c_GetRmaWinRef.errcheck = HandleErrorCode
    rmaWinRef = RmaWinRef()
    c_GetRmaWinRef(attributeList, attribute, ctypes.byref(rmaWinRef))
    return rmaWinRef.value

def AttributeList_GetSourceCodeLocationRef(attributeList, attribute):
    c_GetSourceCodeLocationRef = conf.lib.OTF2_AttributeList_GetSourceCodeLocationRef
    c_GetSourceCodeLocationRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(SourceCodeLocationRef) ]
    c_GetSourceCodeLocationRef.restype = ErrorCode
    c_GetSourceCodeLocationRef.errcheck = HandleErrorCode
    sourceCodeLocationRef = SourceCodeLocationRef()
    c_GetSourceCodeLocationRef(attributeList, attribute, ctypes.byref(sourceCodeLocationRef))
    return sourceCodeLocationRef.value

def AttributeList_GetCallingContextRef(attributeList, attribute):
    c_GetCallingContextRef = conf.lib.OTF2_AttributeList_GetCallingContextRef
    c_GetCallingContextRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(CallingContextRef) ]
    c_GetCallingContextRef.restype = ErrorCode
    c_GetCallingContextRef.errcheck = HandleErrorCode
    callingContextRef = CallingContextRef()
    c_GetCallingContextRef(attributeList, attribute, ctypes.byref(callingContextRef))
    return callingContextRef.value

def AttributeList_GetInterruptGeneratorRef(attributeList, attribute):
    c_GetInterruptGeneratorRef = conf.lib.OTF2_AttributeList_GetInterruptGeneratorRef
    c_GetInterruptGeneratorRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(InterruptGeneratorRef) ]
    c_GetInterruptGeneratorRef.restype = ErrorCode
    c_GetInterruptGeneratorRef.errcheck = HandleErrorCode
    interruptGeneratorRef = InterruptGeneratorRef()
    c_GetInterruptGeneratorRef(attributeList, attribute, ctypes.byref(interruptGeneratorRef))
    return interruptGeneratorRef.value

def AttributeList_GetIoFileRef(attributeList, attribute):
    c_GetIoFileRef = conf.lib.OTF2_AttributeList_GetIoFileRef
    c_GetIoFileRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(IoFileRef) ]
    c_GetIoFileRef.restype = ErrorCode
    c_GetIoFileRef.errcheck = HandleErrorCode
    ioFileRef = IoFileRef()
    c_GetIoFileRef(attributeList, attribute, ctypes.byref(ioFileRef))
    return ioFileRef.value

def AttributeList_GetIoHandleRef(attributeList, attribute):
    c_GetIoHandleRef = conf.lib.OTF2_AttributeList_GetIoHandleRef
    c_GetIoHandleRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(IoHandleRef) ]
    c_GetIoHandleRef.restype = ErrorCode
    c_GetIoHandleRef.errcheck = HandleErrorCode
    ioHandleRef = IoHandleRef()
    c_GetIoHandleRef(attributeList, attribute, ctypes.byref(ioHandleRef))
    return ioHandleRef.value

def AttributeList_GetLocationGroupRef(attributeList, attribute):
    c_GetLocationGroupRef = conf.lib.OTF2_AttributeList_GetLocationGroupRef
    c_GetLocationGroupRef.argtypes = [ ctypes.POINTER(AttributeList), AttributeRef, ctypes.POINTER(LocationGroupRef) ]
    c_GetLocationGroupRef.restype = ErrorCode
    c_GetLocationGroupRef.errcheck = HandleErrorCode
    locationGroupRef = LocationGroupRef()
    c_GetLocationGroupRef(attributeList, attribute, ctypes.byref(locationGroupRef))
    return locationGroupRef.value

def AttributeList_GetAttributeByIndex(attributeList, index, attribute = AttributeRef(),
                                      attributeType = Type(),attributeValue = AttributeValue()):
    c_GetAttributeByIndex = conf.lib.OTF2_AttributeList_GetAttributeByIndex
    c_GetAttributeByIndex.argtypes = [ ctypes.POINTER(AttributeList), ctypes.c_uint32, ctypes.POINTER(AttributeRef), ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_GetAttributeByIndex.restype = ErrorCode
    c_GetAttributeByIndex.errcheck = HandleErrorCode
    c_GetAttributeByIndex(attributeList, index, ctypes.byref(attribute), ctypes.byref(attributeType), ctypes.byref(attributeValue))
    return (attribute.value, attributeType, attributeValue)

def AttributeList_PopAttribute(attributeList, attribute = AttributeRef(),
                               attributeType = Type(), attributeValue = AttributeValue()):
    c_PopAttribute = conf.lib.OTF2_AttributeList_PopAttribute
    c_PopAttribute.argtypes = [ ctypes.POINTER(AttributeList), ctypes.POINTER(AttributeRef), ctypes.POINTER(Type), ctypes.POINTER(AttributeValue) ]
    c_PopAttribute.restype = ErrorCode
    c_PopAttribute.errcheck = HandleErrorCode
    c_PopAttribute(attributeList, ctypes.byref(attribute), ctypes.byref(attributeType), ctypes.byref(attributeValue))
    return (attribute.value, attributeType, attributeValue)

def AttributeList_GetNumberOfElements(attributeList):
    c_GetNumberOfElements = conf.lib.OTF2_AttributeList_GetNumberOfElements
    c_GetNumberOfElements.argtype = [ ctypes.POINTER(AttributeList) ]
    c_GetNumberOfElements.restype = ctypes.c_uint32
    return c_GetNumberOfElements(attributeList)

__all__ = [
    'AttributeList',
    'AttributeList_New',
    'AttributeList_Delete',
    'AttributeList_AddAttribute',
    'AttributeList_AddString',
    'AttributeList_AddUint8',
    'AttributeList_GetUint8',
    'AttributeList_AddUint16',
    'AttributeList_GetUint16',
    'AttributeList_AddUint32',
    'AttributeList_GetUint32',
    'AttributeList_AddUint64',
    'AttributeList_GetUint64',
    'AttributeList_AddInt8',
    'AttributeList_GetInt8',
    'AttributeList_AddInt16',
    'AttributeList_GetInt16',
    'AttributeList_AddInt32',
    'AttributeList_GetInt32',
    'AttributeList_AddInt64',
    'AttributeList_GetInt64',
    'AttributeList_AddFloat',
    'AttributeList_GetFloat',
    'AttributeList_AddDouble',
    'AttributeList_GetDouble',
    'AttributeList_RemoveAttribute',
    'AttributeList_RemoveAllAttributes',
    'AttributeList_TestAttributeByID',
    'AttributeList_GetAttributeByID',
    'AttributeList_AddStringRef',
    'AttributeList_GetStringRef',
    'AttributeList_AddAttributeRef',
    'AttributeList_GetAttributeRef',
    'AttributeList_AddLocationRef',
    'AttributeList_GetLocationRef',
    'AttributeList_AddRegionRef',
    'AttributeList_GetRegionRef',
    'AttributeList_AddGroupRef',
    'AttributeList_GetGroupRef',
    'AttributeList_AddMetricRef',
    'AttributeList_GetMetricRef',
    'AttributeList_AddCommRef',
    'AttributeList_GetCommRef',
    'AttributeList_AddParameterRef',
    'AttributeList_GetParameterRef',
    'AttributeList_AddRmaWinRef',
    'AttributeList_GetRmaWinRef',
    'AttributeList_AddSourceCodeLocationRef',
    'AttributeList_GetSourceCodeLocationRef',
    'AttributeList_AddCallingContextRef',
    'AttributeList_GetCallingContextRef',
    'AttributeList_AddInterruptGeneratorRef',
    'AttributeList_GetInterruptGeneratorRef',
    'AttributeList_AddIoFileRef',
    'AttributeList_GetIoFileRef',
    'AttributeList_AddIoHandleRef',
    'AttributeList_GetIoHandleRef',
    'AttributeList_AddLocationGroupRef',
    'AttributeList_GetLocationGroupRef',
    'AttributeList_GetString',
    'AttributeList_GetAttributeByIndex',
    'AttributeList_PopAttribute',
    'AttributeList_GetNumberOfElements'
]



import os
import os.path
import unittest
from shutil import rmtree
from tempfile import mkdtemp
import time

import otf2

from otf2_writer import generate_trace
from otf2_writer_cct import generate_trace as generate_trace_cct

TIMER_GRANULARITY = 1000000


def t():
    return int(round(time.time() * TIMER_GRANULARITY))


class TestOTF2UserData(unittest.TestCase):

    archive_name = "otf2_test_trace"

    def setUp(self):
        self.old_cwd = os.getcwd()
        self.tmp_dirname = mkdtemp(prefix=os.path.basename(os.path.abspath(__file__))[:-3] + '_tmp', dir=self.old_cwd)
        os.chdir(self.tmp_dirname)

    def tearDown(self):
        os.chdir(self.old_cwd)
        if os.getenv('KEEP_TEST_OUTPUT', '') != '':
            print(self.tmp_dirname)
        else:
            rmtree(self.tmp_dirname)

    def read_trace(self, archive_name, expected_number_of_events):
        with otf2.reader.open(archive_name) as trace:
            count = 0
            for location, event in trace.events:
                count += 1

            self.assertEqual(count, expected_number_of_events)

            self.assertEqual(len(self.definitions.regions), len(trace.definitions.regions))
            for written_region in self.definitions.regions:
                for read_region in trace.definitions.regions:
                    # there should be exactly one region
                    self.assertEqual(written_region.name, read_region.name)

            self.assertEqual(len(self.definitions.strings), len(trace.definitions.strings))
            for written_string in self.definitions.strings:
                self.assertTrue(written_string in trace.definitions.strings)

    def test_integrity(self):
        self.definitions = generate_trace(self.tmp_dirname)
        self.read_trace(os.path.join(self.tmp_dirname, "traces.otf2"), 4)


    def test_integrity_cct(self):
        self.definitions = generate_trace_cct(self.tmp_dirname)
        self.read_trace(os.path.join(self.tmp_dirname, "traces.otf2"), 5)


if __name__ == '__main__':
    unittest.main()



import ctypes

from .Config import conf, StrParam
from .ErrorCodes import ErrorCode, HandleErrorCode
from .GeneralDefinitions import LocationRef
from .SnapReaderCallbacks import SnapReaderCallbacks


class SnapReader(ctypes.Structure):
    pass

def SnapReader_GetLocationID(reader):
    c_GetLocationID = conf.lib.OTF2_SnapReader_GetLocationID
    c_GetLocationID.argtypes = [ ctypes.POINTER(SnapReader), ctypes.POINTER(LocationRef) ]
    c_GetLocationID.restype = ErrorCode
    c_GetLocationID.errcheck = HandleErrorCode
    location = LocationRef()
    c_GetLocationID(reader, ctypes.byref(location))
    return location.value

def SnapReader_SetCallbacks(reader, callbacks, userData):
    c_SetCallbacks = conf.lib.OTF2_SnapReader_SetCallbacks
    c_SetCallbacks.argtypes = [ ctypes.POINTER(SnapReader), ctypes.POINTER(SnapReaderCallbacks), ctypes.py_object ]
    c_SetCallbacks.restype = ErrorCode
    c_SetCallbacks.errcheck = HandleErrorCode
    c_SetCallbacks(reader, callbacks, ctypes.py_object(userData))

def SnapReader_Seek(reader, req_time):
    c_Seek = conf.lib.OTF2_SnapReader_Seek
    c_Seek.argtypes = [ ctypes.POINTER(SnapReader), ctypes.c_uint64, ctypes.POINTER(ctypes.c_int) ]
    c_Seek.restype = ErrorCode
    c_Seek.errcheck = HandleErrorCode
    found = ctypes.c_int()
    c_Seek(reader, req_time, ctypes.byref(found))
    return found.value

def SnapReader_ReadSnapshots(reader, recordsToRead):
    c_ReadSnapshots = conf.lib.OTF2_SnapReader_ReadSnapshots
    c_ReadSnapshots.argtypes = [ ctypes.POINTER(SnapReader), ctypes.c_uint64, ctypes.POINTER(ctypes.c_uint64) ]
    c_ReadSnapshots.restype = ErrorCode
    c_ReadSnapshots.errcheck = HandleErrorCode
    recordsRead = ctypes.c_uint64()
    c_ReadSnapshots(reader, recordsToRead, ctypes.byref(recordsRead))
    return recordsRead.value

__all__ = [
    'SnapReader',
    'SnapReader_GetLocationID',
    'SnapReader_ReadSnapshots',
    'SnapReader_Seek',
    'SnapReader_SetCallbacks'
]



import re
from itertools import chain
from collections import namedtuple
from numbers import Number

import _otf2

from .attribute_value import AttributeValue
from .enums import RegionRole, Paradigm, RegionFlag, LocationType, LocationGroupType, MetricType, \
    MetricMode, MetricScope, MetricOccurrence, RecorderKind, GroupType, GroupFlag, Type, Base, \
    Undefined, ParameterType, SystemTreeDomain, CartPeriodicity, InterruptGeneratorMode, \
    ParadigmClass, IoParadigmClass, IoParadigmFlag, IoParadigmProperty, \
    IoHandleFlag, IoAccessMode, IoStatusFlag, CommFlag, RmaWinFlag
from . import enums  # Needed for namespace issue with Paradigm and ParadigmProperty
from .error import TraceReaderError, TraceWriterError


class DefinitionField(namedtuple("DefinitionField", ['name', 'type', 'private_name', 'default'])):
    def check_value(self, value):
        if value is None:
            return None

        expected_type = self.type

        # Special case: Strings are supposed to be created as native strings
        # instead of String definitions
        if expected_type is String:
            expected_type = str

        if isinstance(value, expected_type):
            return value

        if expected_type is tuple:
            return tuple(value)

        raise TypeError("Wrong type for {}, expected {}, actual {}"
                        .format(self.name, expected_type, type(value)))


def _generate_key(value):
    if isinstance(value, tuple):
        return tuple(_generate_key(e) for e in value)
    if isinstance(value, _RefDefinition):
        return type(value), value._ref
    return value


no_default = object()
self_type = object()


def _to_snake(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()

def _pluralize(name):
    if name[-1] == 's' and not name[-2] == 's':
        return name
    if name[-2:] == 'ss':
        return name + 'es'
    if name[-1] == 'y':
        return name[:-1] + 'ies'
    else:
        return name + 's'

def _make_property(field):
    """
    Create properties to access the private name
    """
    def getter(self):
        return getattr(self, field.private_name)

    def setter(self, value):
        self._set_field(field, value)

    return property(getter, setter)


definitions = []


def _generate_doc(name, field_descriptions):
    doc = ""
    for field in field_descriptions:
        if isinstance(field.type, tuple):
            field_type = field.type[0]
        else:
            field_type = field.type

        type_name = field_type.__name__
        if type_name == "String":
            module = ""
            type_name = "str"
        elif field_type.__module__ == "__builtin__":
            module = ""
        elif field_type.__module__ == "_otf2.GeneralDefinitions":
            module = "otf2."
        elif field_type.__module__ == "_otf2.Definitions":
            module = "otf2."
        elif type_name == "AttributeValue":
            module = ""
            type_name = "value_type"
        else:
            module = "{}.".format(field_type.__module__)

        doc += ":param {module}{type_name} {field.name}: ".format(field=field, module=module,
                                                                  type_name=type_name)
        if field_type == Type and field.default is None:
            doc += "The type is inferred from the current value and should " \
                   "not be provided by the user."
        elif field.default is no_default:
            doc += "required argument"
        else:
            doc += "default value {!r}".format(field.default)
        doc += "\n"

    return doc


class _DefinitionMeta(type):
    def __new__(mcls, name, bases, namespace):
        fields = namespace.get('_fields', ())
        namespace['__slots__'] = \
            tuple("_" + field[0] for field in fields) \
            + namespace.get('__slots__', ())

        namespace.setdefault('_registry_name', '_' + _pluralize(_to_snake(name)))

        clazz = type.__new__(mcls, name, bases, namespace)

        field_descriptions = []
        field_names = []
        for field in fields:
            try:
                field_name, default, field_type = field
            except ValueError:
                field_name, default = field
                # Heuristic to guess a good type.
                field_type = type(default)
                # String is special
                if field_type is str:
                    field_type = String
                elif isinstance(default, int):
                    field_type = int

            if field_type is self_type:
                field_type = clazz
            # Avoid stupid bugs
            assert isinstance(field_type, type) or all(isinstance(t, type) for t in field_type)
            assert not isinstance(default, type)

            private_name = "_" + field_name
            description = DefinitionField(field_name, field_type, private_name, default)
            field_descriptions.append(description)
            field_names.append(field_name)
            if not hasattr(clazz, field_name):
                setattr(clazz, field_name, _make_property(description))

        for base in bases:
            field_descriptions = list(getattr(base, '_fields', ())) + field_descriptions

        clazz._fields = tuple(field_descriptions)
        doc = _generate_doc(name, field_descriptions)
        if clazz.__doc__:
            doc = clazz.__doc__ + "\n" + doc
        clazz.__doc__ = doc
        if '_abstract' not in namespace and not name == "NewBase":
            definitions.append(clazz)
        return clazz


class _RefDefinitionMeta(_DefinitionMeta):
    def __new__(mcls, name, bases, namespace):
        if '_abstract' not in namespace and not name == "NewBase":
            namespace.setdefault('_ref_name', _to_snake(name))
            namespace.setdefault('_registry_name', '_' + _pluralize(namespace['_ref_name']))
            namespace.setdefault('undefined_ref', getattr(Undefined, namespace['_ref_name'].upper()))
            try:
                namespace.setdefault('type_id', getattr(Type, namespace['_ref_name'].upper()))
            except:
                pass

        return _DefinitionMeta.__new__(mcls, name, bases, namespace)


class _Definition(metaclass=_DefinitionMeta):
    """
    Any definition. It is attached to a registry.
    """
    __slots__ = '_registry',
    _abstract = True

    def __init__(self, registry, *args, **kwargs):
        self._registry = registry

        if len(self._fields) < len(args) + len(kwargs):
            raise TypeError("Too many arguments.")

        for idx, field in enumerate(self._fields):
            try:
                value = args[idx]
            except IndexError:
                if field.name in kwargs:
                    value = kwargs.pop(field.name)
                elif field.default is not no_default:
                    value = field.default
                    # Avoid mutating the default itself...
                    if type(value) is tuple:
                        value = value.copy()
                else:
                    raise TypeError("Required parameter '{}' missing.".format(field.name))

            value = field.check_value(value)
            setattr(self, field.private_name, value)
        if kwargs:
            raise TypeError("Unused argument.")

    @staticmethod
    def _construct_arg(registry, def_type, val):
        if isinstance(def_type, type) and issubclass(def_type, _RefDefinition):
            return registry._get(def_type, val)
        return val

    @classmethod
    def _construct(cls, registry, *args):
        """ Construct a definition based on the parameters of a callback """
        conv_args = [cls._construct_arg(registry, field.type, arg)
                     for arg, field in zip(args, cls._fields)]
        return cls(registry._registry_for_type(cls), *conv_args)

    def _set_field(self, field, value):
        if self._registry is None:
            raise AttributeError("This object is immutable")
        old_key = self._key
        value = field.check_value(value)
        setattr(self, field.private_name, value)
        self._registry._update(old_key, self)

    @classmethod
    def _new_key(cls, registry, *args, **kwargs):
        # Create fake object without registry and ref just to get a key
        return cls(registry, None, *args, **kwargs)._key

    @property
    def _key(self):
        """
        A key identifies the actual values of a definition.
        It shall be valid for hashing and comparison.
        """
        # Possible optimization: Cache key!
        field_keys = [_generate_key(getattr(self, field.private_name)) for field in self._fields]
        # Put the class first when mixing different definition classes
        return tuple(chain((type(self),), field_keys))

    @staticmethod
    def _write_arg(writer, definition_registry, obj_type, obj):
        if obj is None:
            return obj_type.undefined_ref
        if obj_type is String:
            return definition_registry.strings.get_ref(obj, writer)
        if isinstance(obj_type, type) and issubclass(obj_type, _RefDefinition):
            return obj._ref
        if obj_type is AttributeValue:
            return obj.as_otf2(definition_registry, writer)
        return obj

    def write(self, writer, def_registry):
        if def_registry._registry_for_type(type(self)) is not self._registry:
            raise ValueError("Trying to write a definition with the wrong writer.")
        args = [self._write_arg(writer, def_registry, field.type,
                                getattr(self, field.private_name))
                for field in self._fields]
        writer.write(self, *args)

    def _str_fields(self):
        return ", ".join("'{}': {!r}".format(field.name, getattr(self, field.name))
                         for field in self._fields)

    def __str__(self):
        return "{}: {}".format(type(self).__name__, self._str_fields())


class _RefDefinition(_Definition, metaclass=_RefDefinitionMeta):
    """
    A Definition with a ref (id) and a corresponding IdRegistry that manages this definition.
    """
    __slots__ = '_ref', '_properties'
    _abstract = True

    def __init__(self, registry, ref, *args, **kwargs):
        """ The constructor of a Definition is called by a registry"""
        self._ref = ref
        self._properties = {}
        super(_RefDefinition, self).__init__(registry, *args, **kwargs)

    @classmethod
    def _construct(cls, registry, ref, *args):
        """ Construct a definition based on the parameters of a callback """
        conv_args = [cls._construct_arg(registry, field.type, arg)
                     for arg, field in zip(args, cls._fields)]
        return cls(registry._registry_for_type(cls), ref, *conv_args)

    def write(self, writer, def_registry):
        if def_registry._registry_for_type(type(self)) is not self._registry:
            raise ValueError("Trying to write a definition with the wrong writer.")
        args = [self._write_arg(writer, def_registry, field.type,
                                getattr(self, field.private_name))
                for field in self._fields]
        writer.write(self, self._ref, *args)

    def __getitem__(self, key):
        return self._properties[key].value

    def __setitem__(self, key, value):
        self._properties[key].value = value

    def _add_property(self, key, property):
        assert(key not in self._properties)
        self._properties[key] = property

    def __str__(self):
        return "{} [{}]: {}".format(type(self).__name__, self._ref, self._str_fields())

    def __repr__(self):
        return "{} [{}]".format(type(self).__name__, self._ref)


class String(_RefDefinition):
    _fields = ('name', no_default, str),


class _NamedRefDefinition(_RefDefinition):
    """
    A definition with a name field as the first argument which is enforced to be of type str.
    """
    _fields = ('name', no_default, String),
    _abstract = True

    def __init__(self, registry, ref, name=None, *args, **kwargs):
        if name is None:
            name = "{} #{}".format(self.__class__.__name__, ref)
        super(_NamedRefDefinition, self).__init__(registry, ref, name, *args, **kwargs)

    def __repr__(self):
        return "{} [{}] '{}'".format(type(self).__name__, self._ref, self.name)


class _PropertyDefinition(_Definition):
    """
    Common base class for property definitions.
    All derived classes must have _fields[2] = 'type', _fields[3] = 'value'
    """
    _abstract = True

    def __init__(self, registry, arg0, arg1, type_id=None, value=None):
        if value is None:
            raise TypeError("Missing argument 'value' for PropertyDefinition.")
        if type_id is None:
            type_id = get_type_id(value)
        if not isinstance(value, AttributeValue):
            value = AttributeValue(value, type_id)
        if type_id != value.type_id:
            raise TypeError("Conflicting AttributeValue type ids: {} != {}".format(type_id,
                                                                                   value.type_id))
        super(_PropertyDefinition, self).__init__(registry, arg0, arg1, type_id, value)

        if isinstance(arg0, _RefDefinition):
            if isinstance(arg1, Parameter):
                # Basically any _NamedRefDefinition would be ok, but there's no use case currently
                key = arg1.name
            elif isinstance(arg1, str):
                key = arg1
            else:
                raise TypeError("Invalid type for name (can be string or Parameter)")

            # Adds property to referenced definitons 'arg0' under name 'arg1'
            arg0._add_property(key, self)

    @classmethod
    def _construct(cls, registry, arg0, arg1, type_id, value):
        value = AttributeValue._construct(registry, value, type_id)
        return super(_PropertyDefinition, cls)._construct(registry, arg0, arg1, type_id, value)

    @property
    def type(self):
        return self._value.type_id

    @property
    def value(self):
        return self._value.value

    @value.setter
    def value(self, new_value):
        self._value.set(new_value)

    def write(self, writer, def_registry):
        self._type = self._value.type_id
        super(_PropertyDefinition, self).write(writer, def_registry)


class ClockProperties(_Definition):
    # We don't want numbers.Number here.
    _fields = (('timer_resolution', no_default, int),
               ('global_offset', 0, int),
               ('trace_length', 0, int),
               ('realtime_timestamp', Undefined.TIMESTAMP.value, int))


class Attribute(_NamedRefDefinition):
    type_id = Type.ATTRIBUTE
    _fields = (('description', ''),
               ('type', Type.STRING))


class SystemTreeNode(_NamedRefDefinition):
    """
    .. Note:: The root of the tree should have the parent `None`
    """
    _fields = (('class_name', ''),
               ('parent', None, self_type))


class LocationGroup(_NamedRefDefinition):
    _fields = (('location_group_type', LocationGroupType.PROCESS),
               ('system_tree_parent', no_default, SystemTreeNode),
               ('creating_location_group', None, self_type))


class Location(_NamedRefDefinition):
    """
    .. Note::

        The number of events is managed automatically when writing traces.
        Do not use the attribute when writing.

    """
    _fields = (('type', LocationType.CPU_THREAD),
               ('number_of_events', 0),
               ('group', no_default, LocationGroup))
    __slots__ = '_number_of_events_written',

    def write(self, writer, def_registry):
        try:
            self._number_of_events = self._number_of_events_written
        except AttributeError:
            # If no writer was opened for this definition, the attribute won't be assigned.
            pass
        super(Location, self).write(writer, def_registry)


class Region(_NamedRefDefinition):
    _fields = (('canonical_name', None, String),
               ('description', ''),
               ('region_role', RegionRole.FUNCTION),
               ('paradigm', enums.Paradigm.NONE),
               ('region_flags', RegionFlag.NONE),
               ('source_file', None, String),
               ('begin_line_number', 0),
               ('end_line_number', 0))

    def __init__(self, registry, ref, name=None, canonical_name=None, *args, **kwargs):
        if name is None:
            name = "{} #{}".format(self.__class__.__name__, ref)
        if canonical_name is None:
            canonical_name = name
        super(Region, self).__init__(registry, ref, name, canonical_name, *args, **kwargs)


class Callsite(_RefDefinition):
    _fields = (('source_file', None, String),
               ('line_number', 0),
               ('entered_region', no_default, Region),
               ('left_region', no_default, Region))


class Callpath(_RefDefinition):
    """
    .. Note:: Use `None` as parent to indicate that this is the topmost region in the Callpath.
    """
    _fields = (('parent', no_default, self_type),
               ('region', no_default, Region))


class Group(_NamedRefDefinition):
    """
    Groups of type COMM_GROUP may be constructed from a sequence of either ranks or location
    objects as members argument.
    All other group types must be constructed using a sequence of their respective Definition
    objects.
    The members property is always a sequence of Definition objects, even when the Group is
    constructed using a sequence of ranks.
    """
    _fields = (('group_type', no_default, GroupType),
               ('paradigm', enums.Paradigm.NONE),
               ('group_flags', GroupFlag.NONE),
               ('members', no_default, tuple))

    def __init__(self, registry, ref, name=None, group_type=no_default,
                 paradigm=enums.Paradigm.NONE, group_flags=GroupFlag.NONE, members=no_default):
        if group_type is no_default:
            raise TypeError("Required parameter 'group_type' missing.")
        if members is no_default:
            raise TypeError("Required parameter 'members' missing.")
        if group_type == GroupType.COMM_GROUP and len(members) > 0:
            if isinstance(members[0], Location):
                if not all(isinstance(l, Location) for l in members):
                    raise TypeError("members for a COMM_GROUP must be either all Locations or "
                                    "all ranks")
            else:
                parent_group = None

                for group in registry:
                    if group.paradigm == paradigm and group.group_type == GroupType.COMM_LOCATIONS:
                        parent_group = group

                if parent_group is None:
                    raise TraceReaderError(
                        "Missing referenced group for sup-group: '{}' #{}".format(name, ref))

                members = tuple(parent_group.location(mref) for mref in members)

        super(Group, self).__init__(registry, ref, name, group_type, paradigm, group_flags,
                                    members)


    # Group is special:
    # It converts a a _field between list of Definitions and list of references
    @classmethod
    def _construct(cls, registry, ref, name, group_type, paradigm, group_flags, member_refs):
        if group_type == GroupType.LOCATIONS:
            member_reg = registry.locations
        elif group_type == GroupType.REGIONS:
            member_reg = registry.regions
        elif group_type == GroupType.METRIC:
            member_reg = registry.metrics
        elif group_type == GroupType.COMM_LOCATIONS:
            member_reg = registry.locations
        elif group_type == GroupType.COMM_GROUP:
            # Defer rank->location conversion to normal constructor
            return cls(registry._registry_for_type(cls), ref, registry.strings[name], group_type,
                       paradigm, group_flags, member_refs)
        elif group_type == GroupType.COMM_SELF:
            member_reg = None
        else:
            raise TypeError("Unknown group_type: {}".format(group_type))

        members = [member_reg[mref] for mref in member_refs]

        return cls(registry._registry_for_type(cls), ref, registry.strings[name], group_type,
                   paradigm, group_flags, members)

    def write(self, writer, def_registry):
        if def_registry._registry_for_type(type(self)) is not self._registry:
            raise ValueError("Trying to write a definition with the wrong writer.")

        args = [self._write_arg(writer, def_registry, field.type,
                                getattr(self, field.private_name))
                for field in self._fields]

        if self._group_type == GroupType.COMM_GROUP:
            super_group_members = None

            for group in def_registry.groups:
                if group.paradigm == self._paradigm and group.group_type == GroupType.COMM_LOCATIONS:
                    super_group_members = group.members

            if super_group_members is None:
                raise TraceWriterError(
                    "Missing referenced group for sup-group: '{}' #{}".format(self._name, self._ref))

            args[-1] = [super_group_members.index(member) for member in self._members]

        writer.write(self, self._ref, *args)

    @staticmethod
    def _write_arg(writer, definition_registry, obj_type, obj):
        if obj_type is tuple:
            return tuple(m._ref for m in obj)
        return _NamedRefDefinition._write_arg(writer, definition_registry, obj_type, obj)

    def rank(self, location):
        """
        Get the rank / index of a given :py:class:`Location` or any other contained Definition with
        respect to this group.
        """
        # TODO optimize with bidirectional map
        return self.members.index(location)

    def location(self, rank):
        """
        Get the :py:class:`Location` or other Definition object for a given rank within this group
        """
        return self.members[rank]


class MetricMember(_NamedRefDefinition):
    _fields = (('description', ''),
               ('metric_type', MetricType.OTHER),
               ('metric_mode', MetricMode.ABSOLUTE_POINT),
               ('value_type', Type.DOUBLE),
               ('base', Base.DECIMAL),
               ('exponent', 0),
               ('unit', ''))

    def _convert_value(self, value):
        union_value = _otf2.MetricValue()
        if self.value_type == Type.UINT64:
            union_value.unsigned_int = int(value)
        elif self.value_type == Type.INT64:
            union_value.signed_int = int(value)
        elif self.value_type == Type.DOUBLE:
            union_value.floating_point = float(value)
        else:
            raise TypeError("Cannot convert to metric value type.")
        return union_value

    def _convert_union(self, union_value):
        """
        :param union_value: from low level, is actually a ctypes.Union
        :return: nice native python value of type corresponding to our value_type
        """
        if self.value_type == Type.UINT64:
            return union_value.unsigned_int
        elif self.value_type == Type.INT64:
            return union_value.signed_int
        elif self.value_type == Type.DOUBLE:
            return union_value.floating_point
        else:
            raise TypeError("Cannot convert to metric value type.")


class Metric(_RefDefinition):
    """
    Abstract base class for :py:class:`MetricClass` and :py:class:`MetricInstance`
    definitions.
    """
    _ref_name = 'metric'


class MetricClass(Metric):
    _ref_name = 'metric'
    _fields = (('members', no_default, tuple),
               ('occurrence', MetricOccurrence.ASYNCHRONOUS),
               ('recorder_kind', RecorderKind.UNKNOWN))

    # MetricClass is special:
    # It converts a a _field between tuple of Definitions and list of references
    @classmethod
    def _construct(cls, registry, ref, member_refs, occurence, recorder_kind):
        members = tuple(registry.metric_members[member_ref] for member_ref in member_refs)
        return super(MetricClass, cls)._construct(registry, ref, members, occurence, recorder_kind)

    @staticmethod
    def _write_arg(writer, definition_registry, obj_type, obj):
        if obj_type is tuple:
            return [m._ref for m in obj]
        return _RefDefinition._write_arg(writer, definition_registry, obj_type, obj)

    @property
    def _type_ids(self):
        return [member.value_type for member in self.members]

    def _convert_values(self, values):
        return [member._convert_value(value)
                for value, member in zip(values, self.members)]

    def _convert_unions(self, unions):
        return [member._convert_union(union)
                for union, member in zip(unions, self.members)]

    @property
    def member(self):
        if len(self.members) != 1:
            raise AttributeError("Trying to read single member from MetricClass with {} "
                                 "members".format(len(self.members)))
        return self.members[0]


class MetricInstance(Metric):
    """
    .. Note::

        The metric_scope is always computed based on the type of the current scope.
        Do not explicitly provide that argument.

    """
    _ref_name = 'metric'
    _fields = (('metric_class', no_default, MetricClass),
               ('recorder', no_default, Location),
               ('metric_scope', None, MetricScope),
               ('scope', no_default, _RefDefinition))

    def __init__(self, registry, ref, metric_class, recorder, metric_scope=None, scope=None):
        if scope is None:
            raise TypeError("Missing argument 'scope' for MetricInstance.")

        if metric_scope is None:
            if isinstance(scope, Location):
                metric_scope = MetricScope.LOCATION
            elif isinstance(scope, LocationGroup):
                metric_scope = MetricScope.LOCATION_GROUP
            elif isinstance(scope, SystemTreeNode):
                metric_scope = MetricScope.SYSTEM_TREE_NODE
            elif isinstance(scope, Group):
                metric_scope = MetricScope.GROUP
            else:
                raise TypeError("Invalid scope type: {}.".format(type(scope)))

        super(MetricInstance, self).__init__(registry, ref, metric_class, recorder, metric_scope, scope)

    @property
    def metric_scope(self):
        try:
            return {
                Location: MetricScope.LOCATION,
                LocationGroup: MetricScope.LOCATION_GROUP,
                SystemTreeNode: MetricScope.SYSTEM_TREE_NODE,
                Group: MetricScope.GROUP,
            }[type(self.scope)]
        except KeyError:
            raise TypeError("Invalid scope type.")

    @classmethod
    def _construct(cls, registry, ref, metric_class, recorder, metric_scope, scope):
        if metric_scope == MetricScope.LOCATION:
            scope_reg = registry.locations
        elif metric_scope == MetricScope.LOCATION_GROUP:
            scope_reg = registry.location_groups
        elif metric_scope == MetricScope.SYSTEM_TREE_NODE:
            scope_reg = registry.system_tree_nodes
        elif metric_scope == MetricScope.GROUP:
            if registry.groups[scope].group_type == GroupType.METRIC:
                raise TypeError("You cannot refer to a group of type metric from a metric. "
                                "Really, think about it, it doesn't make any sense.")
            scope_reg = registry.groups
        else:
            raise TypeError("Unexpected MetricScope")
        return cls(registry._registry_for_type(cls), ref, registry.metrics[metric_class],
                   registry.locations[recorder], metric_scope, scope_reg[scope])

    @property
    def _type_ids(self):
        return self.metric_class._type_ids

    def _convert_values(self, values):
        return self.metric_class._convert_values(values)

    def _convert_unions(self, unions):
        return self.metric_class._convert_unions(unions)

    @property
    def members(self):
        return self.metric_class.members

    @property
    def member(self):
        if len(self.members) != 1:
            raise AttributeError("Trying to read single member from MetricClass with {} "
                                 "members".format(len(self.members)))
        return self.members[0]

    def write(self, writer, def_registry):
        self._metric_scope = self.metric_scope
        super(MetricInstance, self).write(writer, def_registry)


class Comm(_NamedRefDefinition):
    _fields = (('group', no_default, Group),
               ('parent', None, self_type),
               ('flags', CommFlag.NONE))

    def rank(self, location):
        """
        Get the rank / index of a given :py:class:`Location` with respect to this communicator.
        """
        return self.group.rank(location)

    def location(self, rank):
        """
        Get the :py:class:`Location` for a given rank within this communicator
        """
        return self.group.location(rank)


class InterComm(Comm):
    _ref_name = 'comm'
    _fields = (('groupA', no_default, Group),
               ('groupB', no_default, Group),
               ('parent', None, self_type),
               ('flags', CommFlag.NONE))

#    def rank(self, location):
#        """
#        Get the rank / index of a given :py:class:`Location` with respect to this communicator.
#        """
#        return self.group.rank(location)

#    def location(self, rank):
#        """
#        Get the :py:class:`Location` for a given rank within this communicator
#        """
#        return self.group.location(rank)


class Parameter(_NamedRefDefinition):
    _fields = (('parameter_type', ParameterType.STRING),)


class RmaWin(_NamedRefDefinition):
    _fields = (('comm', no_default, Comm),
               ('flags', RmaWinFlag.NONE))

    def rank(self, location):
        """
        Get the rank / index of a given :py:class:`Location` with respect to the communicator
        of this RMA window.
        """
        return self.comm.rank(location)

    def location(self, rank):
        """
        Get the :py:class:`Location` for a given rank within the communicator of this RMA window.
        """
        return self.comm.location(rank)


class MetricClassRecorder(_Definition):
    _fields = (('metric_class', no_default, MetricClass),
               ('recorder', no_default, Location))


class SystemTreeNodeProperty(_PropertyDefinition):
    _fields = (('system_tree_node', no_default, SystemTreeNode),
               ('name', no_default, String),
               ('type', None, Type),
               ('value', no_default, AttributeValue))


class SystemTreeNodeDomain(_Definition):
    _fields = (('system_tree_node', no_default, SystemTreeNode),
               ('system_tree_node_domain', no_default, SystemTreeDomain))


class LocationGroupProperty(_PropertyDefinition):
    _fields = (('location_group', no_default, LocationGroup),
               ('name', no_default, String),
               ('type', None, Type),
               ('value', no_default, AttributeValue))


class LocationProperty(_PropertyDefinition):
    _fields = (('location', no_default, Location),
               ('name', no_default, String),
               ('type', None, Type),
               ('value', no_default, AttributeValue))


class CartDimension(_NamedRefDefinition):
    _fields = (("size", no_default, int),
               ("cart_periodicity", CartPeriodicity.FALSE))


class CartTopology(_NamedRefDefinition):
    """
    .. Note:: Dimensions is a tuple of :class:`otf2.definitions.CartDimension`.
    """
    _fields = (("communicator", no_default, Comm),
               ("dimensions", no_default, tuple))

    @classmethod
    def _construct(cls, registry, ref, name, communicator, dimension_refs):
        dimensions = tuple(registry.cart_dimensions[dimension_ref]
                           for dimension_ref in dimension_refs)
        return cls(registry._registry_for_type(cls), ref, registry.strings[name],
                   registry.comms[communicator], dimensions)

    @staticmethod
    def _write_arg(writer, definition_registry, obj_type, obj):
        if obj_type is tuple:
            return [m._ref for m in obj]
        return _NamedRefDefinition._write_arg(writer, definition_registry, obj_type, obj)


class CartCoordinate(_Definition):
    """
    .. Note:: Coordinates is a tuple of integers.
    """
    _fields = (("topology", no_default, CartTopology),
               ("rank", no_default, int),
               ("coordinates", no_default, tuple))


class SourceCodeLocation(_RefDefinition):
    _fields = (('file', no_default, String),
               ('line_number', 0))


class CallingContext(_RefDefinition):
    """
    .. Note:: Use `None` as parent to indicate that this is the topmost CallingContext
    """
    _fields = (("region", no_default, Region),
               ("source_code_location", None, SourceCodeLocation),
               ("parent", no_default, self_type))


class CallingContextProperty(_PropertyDefinition):
    _fields = (('calling_context', no_default, CallingContext),
               ('name', no_default, String),
               ('type', None, Type),
               ('value', no_default, AttributeValue))


class InterruptGenerator(_NamedRefDefinition):
    _fields = (("interrupt_generator_mode", InterruptGeneratorMode.TIME),
               ("base", Base.DECIMAL),
               ("exponent", 0),
               ("period", no_default, int))


class Paradigm(_Definition):
    """
    .. Note:: This definition has no reference. It describes an enum.
    """
    _fields = (("paradigm", no_default, enums.Paradigm),
               ("name", no_default, String),
               ("paradigm_class", no_default, ParadigmClass))


class ParadigmProperty(_PropertyDefinition):
    """
    .. Warning::

        Does **not** refer to a :class:`otf2.definitions.Paradigm`, but
        :class:`otf2.Paradigm`!

    """
    _fields = (("paradigm", no_default, enums.Paradigm),
               ("property", no_default, enums.ParadigmProperty),
               ("type", None, Type),
               ("value", no_default, AttributeValue))


class IoFile(_NamedRefDefinition):
    """
    Abstract base class for :py:class:`IoRegularFile` and :py:class:`IoDirectory`
    definitions.
    """
    _ref_name = 'io_file'


class IoRegularFile(IoFile):
    _ref_name = 'io_file'
    _fields = (("scope", no_default, SystemTreeNode),)


class IoDirectory(IoFile):
    _ref_name = 'io_file'
    _fields = (("scope", no_default, SystemTreeNode),)


class IoFileProperty(_PropertyDefinition):
    _fields = (("io_file", no_default, IoFile),
               ('name', no_default, String),
               ('type', None, Type),
               ('value', no_default, AttributeValue))


class IoParadigm(_RefDefinition):
    _fields = (("identification", no_default, String),
               ("name", no_default, String),
               ("io_paradigm_class", IoParadigmClass.SERIAL),
               ("io_paradigm_flags", IoParadigmFlag.NONE))
    # Reusing _properties from _RefDefinitions
    # They don't factor into the key

    def __init__(self, registry, ref, identification, name, io_paradigm_class, io_paradigm_flags,
                 properties=None):
        if properties is None:
            properties = {}
        for property, value in properties.items():
            if not isinstance(property, IoParadigmProperty):
                raise TypeError("property_values for IoParadigm must be a dict with keys of type "
                                "IoParadigmProperty. Violating key: {}".format(property))
            if not isinstance(value, AttributeValue):
                properties[property] = AttributeValue(value, get_type_id(value))


        super(IoParadigm, self).__init__(registry, ref, identification, name, io_paradigm_class,
                                         io_paradigm_flags)
        self._properties = properties

    def __setitem__(self, property, value):
        # We can set the properties directly here, even if no AttributeValue is set yet
        if isinstance(value, AttributeValue):
            self._properties[property] = value
        else:
            assert property in self._properties
            self._properties[property].value = value

    @classmethod
    def _construct(cls, registry, ref, identification, name, io_paradigm_class, io_paradigm_flags,
                   properties, types, values):
        property_attribute_values = {property: AttributeValue._construct(registry, value, type_id)
                                     for property, type_id, value
                                     in zip(properties, types, values)}
        return super(IoParadigm, cls)._construct(registry, ref, identification, name,
                                                 io_paradigm_class, io_paradigm_flags,
                                                 property_attribute_values)

    def write(self, writer, def_registry):
        if def_registry._registry_for_type(type(self)) is not self._registry:
            raise ValueError("Trying to write a definition with the wrong writer.")
        args = [self._write_arg(writer, def_registry, field.type,
                                getattr(self, field.private_name))
                for field in self._fields]
        properties_list = list(self._properties.keys())
        types_list = [av.type_id for av in self._properties.values()]
        values_list = [av.as_otf2(def_registry, writer) for av in self._properties.values()]
        writer.write(self, self._ref, *(args + [properties_list, types_list, values_list]))



class IoHandle(_NamedRefDefinition):
    _fields = (("file", no_default, IoFile),
               ("io_paradigm", no_default, IoParadigm),
               ("io_handle_flags", IoHandleFlag.NONE),
               ("comm", None, Comm),
               ("parent", None, self_type))

class IoPreCreatedHandleState(_Definition):
    _fields = (('io_handle', no_default, IoHandle),
               ('io_access_mode', no_default, IoAccessMode),
               ('io_status_flags', IoStatusFlag.NONE))

class CallpathParameter(_PropertyDefinition):
    _fields = (('callpath', no_default, Callpath),
               ('parameter', no_default, Parameter),
               ('type', None, Type),
               ('value', no_default, AttributeValue))

def get_type_id(value):
    if isinstance(value, AttributeValue):
        return value.type_id
    elif isinstance(value, int):
        return Type.INT64
    elif isinstance(value, Number):
        return Type.DOUBLE
    elif isinstance(value, str):
        return Type.STRING
    elif isinstance(value, _RefDefinition) and hasattr(value, 'type_id'):
        return value.type_id
    else:
        raise TypeError("Encountered value with unexpected type for an attribute value: "
                        "'{} ({})'".format(value, type(value)))

__all__ = [definition.__name__ for definition in definitions]
__all__ += ['get_type_id']




''' Generated by OTF2 Template Engine '''

import ctypes
import traceback
import sys

from .Config import conf, StrParam, str_decode
from .ErrorCodes import ErrorCode, HandleErrorCode
from .GeneralDefinitions import *
from .AttributeValue import AttributeValue
from .Definitions import *
from .Callbacks import callback_wrapper


class GlobalDefReaderCallbacks(ctypes.Structure):
    pass

def GlobalDefReaderCallbacks_New():
    c_New = conf.lib.OTF2_GlobalDefReaderCallbacks_New
    c_New.argtypes = []
    c_New.restype = ctypes.POINTER(GlobalDefReaderCallbacks)
    # NOTE: Do not errcheck here. This function returns a value, not an error code
    return c_New()

def GlobalDefReaderCallbacks_Delete(globalDefReaderCallbacks):
    c_Delete = conf.lib.OTF2_GlobalDefReaderCallbacks_Delete
    c_Delete.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks) ]
    c_Delete.restype = None
    return c_Delete(globalDefReaderCallbacks)

def GlobalDefReaderCallbacks_Clear(globalDefReaderCallbacks):
    c_Clear = conf.lib.OTF2_GlobalDefReaderCallbacks_Clear
    c_Clear.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks) ]
    c_Clear.restype = None
    return c_Clear(globalDefReaderCallbacks)

def _callback_wrapper(type, func, convert_args=None):
    def wrapper(userData, *args):
        if userData:
            py_userData = ctypes.cast(userData, ctypes.py_object).value
        else:
            py_userData = None
        try:
            if convert_args is not None:
                args = convert_args(*args)
            ret = func(py_userData, *args)
            if ret is None:
                ret = CALLBACK_SUCCESS
        except:
            sys.stderr.write("An unhandled python exception has occurred in an "
                             "OTF2_GlobalDefReaderCallback:\n")
            sys.stderr.write(traceback.format_exc())
            ret = CALLBACK_ERROR
        return ret.value
    return callback_wrapper(func, wrapper, type)

def _convert_property_union(refered_def, name, type, value):
    union = AttributeValue()
    union._arg_value = value
    return refered_def, name, type, union

_GlobalDefReaderCallback_FP_Unknown = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p)

_GlobalDefReaderCallback_FP_ClockProperties = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, ctypes.c_uint64, ctypes.c_uint64, ctypes.c_uint64, ctypes.c_uint64)

_GlobalDefReaderCallback_FP_Paradigm = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, Paradigm, StringRef, ParadigmClass)

_GlobalDefReaderCallback_FP_ParadigmProperty = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, Paradigm, ParadigmProperty, Type, AttributeValue._arg_type)

_GlobalDefReaderCallback_FP_IoParadigm = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, IoParadigmRef, StringRef, StringRef, IoParadigmClass, IoParadigmFlag, ctypes.c_uint8, ctypes.POINTER(IoParadigmProperty), ctypes.POINTER(Type), ctypes.POINTER(AttributeValue))

_GlobalDefReaderCallback_FP_String = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, StringRef, ctypes.c_char_p)

_GlobalDefReaderCallback_FP_Attribute = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, AttributeRef, StringRef, StringRef, Type)

_GlobalDefReaderCallback_FP_SystemTreeNode = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, SystemTreeNodeRef, StringRef, StringRef, SystemTreeNodeRef)

_GlobalDefReaderCallback_FP_LocationGroup = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, LocationGroupRef, StringRef, LocationGroupType, SystemTreeNodeRef, LocationGroupRef)

_GlobalDefReaderCallback_FP_Location = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, LocationRef, StringRef, LocationType, ctypes.c_uint64, LocationGroupRef)

_GlobalDefReaderCallback_FP_Region = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, RegionRef, StringRef, StringRef, StringRef, RegionRole, Paradigm, RegionFlag, StringRef, ctypes.c_uint32, ctypes.c_uint32)

_GlobalDefReaderCallback_FP_Callsite = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, CallsiteRef, StringRef, ctypes.c_uint32, RegionRef, RegionRef)

_GlobalDefReaderCallback_FP_Callpath = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, CallpathRef, CallpathRef, RegionRef)

_GlobalDefReaderCallback_FP_Group = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, GroupRef, StringRef, GroupType, Paradigm, GroupFlag, ctypes.c_uint32, ctypes.POINTER(ctypes.c_uint64))

_GlobalDefReaderCallback_FP_MetricMember = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, MetricMemberRef, StringRef, StringRef, MetricType, MetricMode, Type, Base, ctypes.c_int64, StringRef)

_GlobalDefReaderCallback_FP_MetricClass = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, MetricRef, ctypes.c_uint8, ctypes.POINTER(MetricMemberRef), MetricOccurrence, RecorderKind)

_GlobalDefReaderCallback_FP_MetricInstance = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, MetricRef, MetricRef, LocationRef, MetricScope, ctypes.c_uint64)

_GlobalDefReaderCallback_FP_Comm = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, CommRef, StringRef, GroupRef, CommRef, CommFlag)

_GlobalDefReaderCallback_FP_Parameter = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, ParameterRef, StringRef, ParameterType)

_GlobalDefReaderCallback_FP_RmaWin = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, RmaWinRef, StringRef, CommRef, RmaWinFlag)

_GlobalDefReaderCallback_FP_MetricClassRecorder = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, MetricRef, LocationRef)

_GlobalDefReaderCallback_FP_SystemTreeNodeProperty = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, SystemTreeNodeRef, StringRef, Type, AttributeValue._arg_type)

_GlobalDefReaderCallback_FP_SystemTreeNodeDomain = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, SystemTreeNodeRef, SystemTreeDomain)

_GlobalDefReaderCallback_FP_LocationGroupProperty = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, LocationGroupRef, StringRef, Type, AttributeValue._arg_type)

_GlobalDefReaderCallback_FP_LocationProperty = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, LocationRef, StringRef, Type, AttributeValue._arg_type)

_GlobalDefReaderCallback_FP_CartDimension = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, CartDimensionRef, StringRef, ctypes.c_uint32, CartPeriodicity)

_GlobalDefReaderCallback_FP_CartTopology = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, CartTopologyRef, StringRef, CommRef, ctypes.c_uint8, ctypes.POINTER(CartDimensionRef))

_GlobalDefReaderCallback_FP_CartCoordinate = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, CartTopologyRef, ctypes.c_uint32, ctypes.c_uint8, ctypes.POINTER(ctypes.c_uint32))

_GlobalDefReaderCallback_FP_SourceCodeLocation = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, SourceCodeLocationRef, StringRef, ctypes.c_uint32)

_GlobalDefReaderCallback_FP_CallingContext = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, CallingContextRef, RegionRef, SourceCodeLocationRef, CallingContextRef)

_GlobalDefReaderCallback_FP_CallingContextProperty = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, CallingContextRef, StringRef, Type, AttributeValue._arg_type)

_GlobalDefReaderCallback_FP_InterruptGenerator = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, InterruptGeneratorRef, StringRef, InterruptGeneratorMode, Base, ctypes.c_int64, ctypes.c_uint64)

_GlobalDefReaderCallback_FP_IoFileProperty = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, IoFileRef, StringRef, Type, AttributeValue._arg_type)

_GlobalDefReaderCallback_FP_IoRegularFile = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, IoFileRef, StringRef, SystemTreeNodeRef)

_GlobalDefReaderCallback_FP_IoDirectory = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, IoFileRef, StringRef, SystemTreeNodeRef)

_GlobalDefReaderCallback_FP_IoHandle = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, IoHandleRef, StringRef, IoFileRef, IoParadigmRef, IoHandleFlag, CommRef, IoHandleRef)

_GlobalDefReaderCallback_FP_IoPreCreatedHandleState = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, IoHandleRef, IoAccessMode, IoStatusFlag)

_GlobalDefReaderCallback_FP_CallpathParameter = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, CallpathRef, ParameterRef, Type, AttributeValue._arg_type)

_GlobalDefReaderCallback_FP_InterComm = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, CommRef, StringRef, GroupRef, GroupRef, CommRef, CommFlag)

def GlobalDefReaderCallbacks_SetUnknownCallback(globalDefReaderCallbacks, unknownCallback):
    c_SetUnknownCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetUnknownCallback
    c_SetUnknownCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_Unknown ]
    c_SetUnknownCallback.restype = ErrorCode
    c_SetUnknownCallback.errcheck = HandleErrorCode
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_Unknown,
                                         unknownCallback)
    c_SetUnknownCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetClockPropertiesCallback(globalDefReaderCallbacks, clockPropertiesCallback):
    c_SetClockPropertiesCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetClockPropertiesCallback
    c_SetClockPropertiesCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_ClockProperties ]
    c_SetClockPropertiesCallback.restype = ErrorCode
    c_SetClockPropertiesCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_ClockProperties,
                                         clockPropertiesCallback,
                                         convert)
    c_SetClockPropertiesCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetParadigmCallback(globalDefReaderCallbacks, paradigmCallback):
    c_SetParadigmCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetParadigmCallback
    c_SetParadigmCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_Paradigm ]
    c_SetParadigmCallback.restype = ErrorCode
    c_SetParadigmCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_Paradigm,
                                         paradigmCallback,
                                         convert)
    c_SetParadigmCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetParadigmPropertyCallback(globalDefReaderCallbacks, paradigmPropertyCallback):
    c_SetParadigmPropertyCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetParadigmPropertyCallback
    c_SetParadigmPropertyCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_ParadigmProperty ]
    c_SetParadigmPropertyCallback.restype = ErrorCode
    c_SetParadigmPropertyCallback.errcheck = HandleErrorCode
    convert = _convert_property_union
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_ParadigmProperty,
                                         paradigmPropertyCallback,
                                         convert)
    c_SetParadigmPropertyCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetIoParadigmCallback(globalDefReaderCallbacks, ioParadigmCallback):
    c_SetIoParadigmCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetIoParadigmCallback
    c_SetIoParadigmCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_IoParadigm ]
    c_SetIoParadigmCallback.restype = ErrorCode
    c_SetIoParadigmCallback.errcheck = HandleErrorCode
    def convert_property(ref, identification, name, io_paradigm_class, io_paradigm_flags, \
                         number_of_properties, properties_array, types_array, values_array):
        properties = [properties_array[i] for i in range(number_of_properties)]
        types = [types_array[i] for i in range(number_of_properties)]
        values = [values_array[i] for i in range(number_of_properties)]
        return ref, identification, name, io_paradigm_class, io_paradigm_flags, properties, \
               types, values
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_IoParadigm,
                                         ioParadigmCallback, convert_property)
    c_SetIoParadigmCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetStringCallback(globalDefReaderCallbacks, stringCallback):
    c_SetStringCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetStringCallback
    c_SetStringCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_String ]
    c_SetStringCallback.restype = ErrorCode
    c_SetStringCallback.errcheck = HandleErrorCode
    def convert_string(ref, name):
        return ref, str_decode(name)
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_String,
                                         stringCallback, convert_string)
    c_SetStringCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetAttributeCallback(globalDefReaderCallbacks, attributeCallback):
    c_SetAttributeCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetAttributeCallback
    c_SetAttributeCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_Attribute ]
    c_SetAttributeCallback.restype = ErrorCode
    c_SetAttributeCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_Attribute,
                                         attributeCallback,
                                         convert)
    c_SetAttributeCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetSystemTreeNodeCallback(globalDefReaderCallbacks, systemTreeNodeCallback):
    c_SetSystemTreeNodeCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetSystemTreeNodeCallback
    c_SetSystemTreeNodeCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_SystemTreeNode ]
    c_SetSystemTreeNodeCallback.restype = ErrorCode
    c_SetSystemTreeNodeCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_SystemTreeNode,
                                         systemTreeNodeCallback,
                                         convert)
    c_SetSystemTreeNodeCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetLocationGroupCallback(globalDefReaderCallbacks, locationGroupCallback):
    c_SetLocationGroupCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetLocationGroupCallback
    c_SetLocationGroupCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_LocationGroup ]
    c_SetLocationGroupCallback.restype = ErrorCode
    c_SetLocationGroupCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_LocationGroup,
                                         locationGroupCallback,
                                         convert)
    c_SetLocationGroupCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetLocationCallback(globalDefReaderCallbacks, locationCallback):
    c_SetLocationCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetLocationCallback
    c_SetLocationCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_Location ]
    c_SetLocationCallback.restype = ErrorCode
    c_SetLocationCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_Location,
                                         locationCallback,
                                         convert)
    c_SetLocationCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetRegionCallback(globalDefReaderCallbacks, regionCallback):
    c_SetRegionCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetRegionCallback
    c_SetRegionCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_Region ]
    c_SetRegionCallback.restype = ErrorCode
    c_SetRegionCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_Region,
                                         regionCallback,
                                         convert)
    c_SetRegionCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetCallsiteCallback(globalDefReaderCallbacks, callsiteCallback):
    c_SetCallsiteCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetCallsiteCallback
    c_SetCallsiteCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_Callsite ]
    c_SetCallsiteCallback.restype = ErrorCode
    c_SetCallsiteCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_Callsite,
                                         callsiteCallback,
                                         convert)
    c_SetCallsiteCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetCallpathCallback(globalDefReaderCallbacks, callpathCallback):
    c_SetCallpathCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetCallpathCallback
    c_SetCallpathCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_Callpath ]
    c_SetCallpathCallback.restype = ErrorCode
    c_SetCallpathCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_Callpath,
                                         callpathCallback,
                                         convert)
    c_SetCallpathCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetGroupCallback(globalDefReaderCallbacks, groupCallback):
    c_SetGroupCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetGroupCallback
    c_SetGroupCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_Group ]
    c_SetGroupCallback.restype = ErrorCode
    c_SetGroupCallback.errcheck = HandleErrorCode
    def convert_group(ref, name, group_type, paradigm, group_flags, number_of_metrics,
                      members_array):
        members = [members_array[i] for i in range(number_of_metrics)]
        return ref, name, group_type, paradigm, group_flags, members
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_Group,
                                         groupCallback, convert_group)
    c_SetGroupCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetMetricMemberCallback(globalDefReaderCallbacks, metricMemberCallback):
    c_SetMetricMemberCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetMetricMemberCallback
    c_SetMetricMemberCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_MetricMember ]
    c_SetMetricMemberCallback.restype = ErrorCode
    c_SetMetricMemberCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_MetricMember,
                                         metricMemberCallback,
                                         convert)
    c_SetMetricMemberCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetMetricClassCallback(globalDefReaderCallbacks, metricClassCallback):
    c_SetMetricClassCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetMetricClassCallback
    c_SetMetricClassCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_MetricClass ]
    c_SetMetricClassCallback.restype = ErrorCode
    c_SetMetricClassCallback.errcheck = HandleErrorCode
    def convert_metric_class(ref, number_of_metrics, metric_members_array, metric_occurrence,
                             recorder_kind):
        metric_members = [metric_members_array[i] for i in range(number_of_metrics)]
        return ref, metric_members, metric_occurrence, recorder_kind
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_MetricClass,
                                         metricClassCallback, convert_metric_class)
    c_SetMetricClassCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetMetricInstanceCallback(globalDefReaderCallbacks, metricInstanceCallback):
    c_SetMetricInstanceCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetMetricInstanceCallback
    c_SetMetricInstanceCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_MetricInstance ]
    c_SetMetricInstanceCallback.restype = ErrorCode
    c_SetMetricInstanceCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_MetricInstance,
                                         metricInstanceCallback,
                                         convert)
    c_SetMetricInstanceCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetCommCallback(globalDefReaderCallbacks, commCallback):
    c_SetCommCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetCommCallback
    c_SetCommCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_Comm ]
    c_SetCommCallback.restype = ErrorCode
    c_SetCommCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_Comm,
                                         commCallback,
                                         convert)
    c_SetCommCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetParameterCallback(globalDefReaderCallbacks, parameterCallback):
    c_SetParameterCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetParameterCallback
    c_SetParameterCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_Parameter ]
    c_SetParameterCallback.restype = ErrorCode
    c_SetParameterCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_Parameter,
                                         parameterCallback,
                                         convert)
    c_SetParameterCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetRmaWinCallback(globalDefReaderCallbacks, rmaWinCallback):
    c_SetRmaWinCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetRmaWinCallback
    c_SetRmaWinCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_RmaWin ]
    c_SetRmaWinCallback.restype = ErrorCode
    c_SetRmaWinCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_RmaWin,
                                         rmaWinCallback,
                                         convert)
    c_SetRmaWinCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetMetricClassRecorderCallback(globalDefReaderCallbacks, metricClassRecorderCallback):
    c_SetMetricClassRecorderCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetMetricClassRecorderCallback
    c_SetMetricClassRecorderCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_MetricClassRecorder ]
    c_SetMetricClassRecorderCallback.restype = ErrorCode
    c_SetMetricClassRecorderCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_MetricClassRecorder,
                                         metricClassRecorderCallback,
                                         convert)
    c_SetMetricClassRecorderCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetSystemTreeNodePropertyCallback(globalDefReaderCallbacks, systemTreeNodePropertyCallback):
    c_SetSystemTreeNodePropertyCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetSystemTreeNodePropertyCallback
    c_SetSystemTreeNodePropertyCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_SystemTreeNodeProperty ]
    c_SetSystemTreeNodePropertyCallback.restype = ErrorCode
    c_SetSystemTreeNodePropertyCallback.errcheck = HandleErrorCode
    convert = _convert_property_union
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_SystemTreeNodeProperty,
                                         systemTreeNodePropertyCallback,
                                         convert)
    c_SetSystemTreeNodePropertyCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetSystemTreeNodeDomainCallback(globalDefReaderCallbacks, systemTreeNodeDomainCallback):
    c_SetSystemTreeNodeDomainCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetSystemTreeNodeDomainCallback
    c_SetSystemTreeNodeDomainCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_SystemTreeNodeDomain ]
    c_SetSystemTreeNodeDomainCallback.restype = ErrorCode
    c_SetSystemTreeNodeDomainCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_SystemTreeNodeDomain,
                                         systemTreeNodeDomainCallback,
                                         convert)
    c_SetSystemTreeNodeDomainCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetLocationGroupPropertyCallback(globalDefReaderCallbacks, locationGroupPropertyCallback):
    c_SetLocationGroupPropertyCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetLocationGroupPropertyCallback
    c_SetLocationGroupPropertyCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_LocationGroupProperty ]
    c_SetLocationGroupPropertyCallback.restype = ErrorCode
    c_SetLocationGroupPropertyCallback.errcheck = HandleErrorCode
    convert = _convert_property_union
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_LocationGroupProperty,
                                         locationGroupPropertyCallback,
                                         convert)
    c_SetLocationGroupPropertyCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetLocationPropertyCallback(globalDefReaderCallbacks, locationPropertyCallback):
    c_SetLocationPropertyCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetLocationPropertyCallback
    c_SetLocationPropertyCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_LocationProperty ]
    c_SetLocationPropertyCallback.restype = ErrorCode
    c_SetLocationPropertyCallback.errcheck = HandleErrorCode
    convert = _convert_property_union
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_LocationProperty,
                                         locationPropertyCallback,
                                         convert)
    c_SetLocationPropertyCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetCartDimensionCallback(globalDefReaderCallbacks, cartDimensionCallback):
    c_SetCartDimensionCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetCartDimensionCallback
    c_SetCartDimensionCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_CartDimension ]
    c_SetCartDimensionCallback.restype = ErrorCode
    c_SetCartDimensionCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_CartDimension,
                                         cartDimensionCallback,
                                         convert)
    c_SetCartDimensionCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetCartTopologyCallback(globalDefReaderCallbacks, cartTopologyCallback):
    c_SetCartTopologyCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetCartTopologyCallback
    c_SetCartTopologyCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_CartTopology ]
    c_SetCartTopologyCallback.restype = ErrorCode
    c_SetCartTopologyCallback.errcheck = HandleErrorCode
    def convert_cart_topology(ref, name, communicator, number_of_dimensions,
                              cart_dimensions_array):
        cart_dimensions = [cart_dimensions_array[i] for i in range(number_of_dimensions)]
        return ref, name, communicator, cart_dimensions
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_CartTopology,
                                         cartTopologyCallback, convert_cart_topology)
    c_SetCartTopologyCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetCartCoordinateCallback(globalDefReaderCallbacks, cartCoordinateCallback):
    c_SetCartCoordinateCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetCartCoordinateCallback
    c_SetCartCoordinateCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_CartCoordinate ]
    c_SetCartCoordinateCallback.restype = ErrorCode
    c_SetCartCoordinateCallback.errcheck = HandleErrorCode
    def convert_cart_coordinate(cart_topology, rank, number_of_dimensions, coordinates_array):
        coordinates = [coordinates_array[i] for i in range(number_of_dimensions)]
        return cart_topology, rank, coordinates
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_CartCoordinate,
                                         cartCoordinateCallback, convert_cart_coordinate)
    c_SetCartCoordinateCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetSourceCodeLocationCallback(globalDefReaderCallbacks, sourceCodeLocationCallback):
    c_SetSourceCodeLocationCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetSourceCodeLocationCallback
    c_SetSourceCodeLocationCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_SourceCodeLocation ]
    c_SetSourceCodeLocationCallback.restype = ErrorCode
    c_SetSourceCodeLocationCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_SourceCodeLocation,
                                         sourceCodeLocationCallback,
                                         convert)
    c_SetSourceCodeLocationCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetCallingContextCallback(globalDefReaderCallbacks, callingContextCallback):
    c_SetCallingContextCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetCallingContextCallback
    c_SetCallingContextCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_CallingContext ]
    c_SetCallingContextCallback.restype = ErrorCode
    c_SetCallingContextCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_CallingContext,
                                         callingContextCallback,
                                         convert)
    c_SetCallingContextCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetCallingContextPropertyCallback(globalDefReaderCallbacks, callingContextPropertyCallback):
    c_SetCallingContextPropertyCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetCallingContextPropertyCallback
    c_SetCallingContextPropertyCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_CallingContextProperty ]
    c_SetCallingContextPropertyCallback.restype = ErrorCode
    c_SetCallingContextPropertyCallback.errcheck = HandleErrorCode
    convert = _convert_property_union
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_CallingContextProperty,
                                         callingContextPropertyCallback,
                                         convert)
    c_SetCallingContextPropertyCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetInterruptGeneratorCallback(globalDefReaderCallbacks, interruptGeneratorCallback):
    c_SetInterruptGeneratorCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetInterruptGeneratorCallback
    c_SetInterruptGeneratorCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_InterruptGenerator ]
    c_SetInterruptGeneratorCallback.restype = ErrorCode
    c_SetInterruptGeneratorCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_InterruptGenerator,
                                         interruptGeneratorCallback,
                                         convert)
    c_SetInterruptGeneratorCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetIoFilePropertyCallback(globalDefReaderCallbacks, ioFilePropertyCallback):
    c_SetIoFilePropertyCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetIoFilePropertyCallback
    c_SetIoFilePropertyCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_IoFileProperty ]
    c_SetIoFilePropertyCallback.restype = ErrorCode
    c_SetIoFilePropertyCallback.errcheck = HandleErrorCode
    convert = _convert_property_union
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_IoFileProperty,
                                         ioFilePropertyCallback,
                                         convert)
    c_SetIoFilePropertyCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetIoRegularFileCallback(globalDefReaderCallbacks, ioRegularFileCallback):
    c_SetIoRegularFileCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetIoRegularFileCallback
    c_SetIoRegularFileCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_IoRegularFile ]
    c_SetIoRegularFileCallback.restype = ErrorCode
    c_SetIoRegularFileCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_IoRegularFile,
                                         ioRegularFileCallback,
                                         convert)
    c_SetIoRegularFileCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetIoDirectoryCallback(globalDefReaderCallbacks, ioDirectoryCallback):
    c_SetIoDirectoryCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetIoDirectoryCallback
    c_SetIoDirectoryCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_IoDirectory ]
    c_SetIoDirectoryCallback.restype = ErrorCode
    c_SetIoDirectoryCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_IoDirectory,
                                         ioDirectoryCallback,
                                         convert)
    c_SetIoDirectoryCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetIoHandleCallback(globalDefReaderCallbacks, ioHandleCallback):
    c_SetIoHandleCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetIoHandleCallback
    c_SetIoHandleCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_IoHandle ]
    c_SetIoHandleCallback.restype = ErrorCode
    c_SetIoHandleCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_IoHandle,
                                         ioHandleCallback,
                                         convert)
    c_SetIoHandleCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetIoPreCreatedHandleStateCallback(globalDefReaderCallbacks, ioPreCreatedHandleStateCallback):
    c_SetIoPreCreatedHandleStateCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetIoPreCreatedHandleStateCallback
    c_SetIoPreCreatedHandleStateCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_IoPreCreatedHandleState ]
    c_SetIoPreCreatedHandleStateCallback.restype = ErrorCode
    c_SetIoPreCreatedHandleStateCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_IoPreCreatedHandleState,
                                         ioPreCreatedHandleStateCallback,
                                         convert)
    c_SetIoPreCreatedHandleStateCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetCallpathParameterCallback(globalDefReaderCallbacks, callpathParameterCallback):
    c_SetCallpathParameterCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetCallpathParameterCallback
    c_SetCallpathParameterCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_CallpathParameter ]
    c_SetCallpathParameterCallback.restype = ErrorCode
    c_SetCallpathParameterCallback.errcheck = HandleErrorCode
    convert = _convert_property_union
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_CallpathParameter,
                                         callpathParameterCallback,
                                         convert)
    c_SetCallpathParameterCallback(globalDefReaderCallbacks, wrapped_callback)

def GlobalDefReaderCallbacks_SetInterCommCallback(globalDefReaderCallbacks, interCommCallback):
    c_SetInterCommCallback = conf.lib.OTF2_GlobalDefReaderCallbacks_SetInterCommCallback
    c_SetInterCommCallback.argtypes = [ ctypes.POINTER(GlobalDefReaderCallbacks), _GlobalDefReaderCallback_FP_InterComm ]
    c_SetInterCommCallback.restype = ErrorCode
    c_SetInterCommCallback.errcheck = HandleErrorCode
    convert = None
    wrapped_callback = _callback_wrapper(_GlobalDefReaderCallback_FP_InterComm,
                                         interCommCallback,
                                         convert)
    c_SetInterCommCallback(globalDefReaderCallbacks, wrapped_callback)

__all__ = [
    'GlobalDefReaderCallbacks',
    'GlobalDefReaderCallbacks_New',
    'GlobalDefReaderCallbacks_Delete',
    'GlobalDefReaderCallbacks_Clear',
    'GlobalDefReaderCallbacks_SetUnknownCallback',
    'GlobalDefReaderCallbacks_SetClockPropertiesCallback',
    'GlobalDefReaderCallbacks_SetParadigmCallback',
    'GlobalDefReaderCallbacks_SetParadigmPropertyCallback',
    'GlobalDefReaderCallbacks_SetIoParadigmCallback',
    'GlobalDefReaderCallbacks_SetStringCallback',
    'GlobalDefReaderCallbacks_SetAttributeCallback',
    'GlobalDefReaderCallbacks_SetSystemTreeNodeCallback',
    'GlobalDefReaderCallbacks_SetLocationGroupCallback',
    'GlobalDefReaderCallbacks_SetLocationCallback',
    'GlobalDefReaderCallbacks_SetRegionCallback',
    'GlobalDefReaderCallbacks_SetCallsiteCallback',
    'GlobalDefReaderCallbacks_SetCallpathCallback',
    'GlobalDefReaderCallbacks_SetGroupCallback',
    'GlobalDefReaderCallbacks_SetMetricMemberCallback',
    'GlobalDefReaderCallbacks_SetMetricClassCallback',
    'GlobalDefReaderCallbacks_SetMetricInstanceCallback',
    'GlobalDefReaderCallbacks_SetCommCallback',
    'GlobalDefReaderCallbacks_SetParameterCallback',
    'GlobalDefReaderCallbacks_SetRmaWinCallback',
    'GlobalDefReaderCallbacks_SetMetricClassRecorderCallback',
    'GlobalDefReaderCallbacks_SetSystemTreeNodePropertyCallback',
    'GlobalDefReaderCallbacks_SetSystemTreeNodeDomainCallback',
    'GlobalDefReaderCallbacks_SetLocationGroupPropertyCallback',
    'GlobalDefReaderCallbacks_SetLocationPropertyCallback',
    'GlobalDefReaderCallbacks_SetCartDimensionCallback',
    'GlobalDefReaderCallbacks_SetCartTopologyCallback',
    'GlobalDefReaderCallbacks_SetCartCoordinateCallback',
    'GlobalDefReaderCallbacks_SetSourceCodeLocationCallback',
    'GlobalDefReaderCallbacks_SetCallingContextCallback',
    'GlobalDefReaderCallbacks_SetCallingContextPropertyCallback',
    'GlobalDefReaderCallbacks_SetInterruptGeneratorCallback',
    'GlobalDefReaderCallbacks_SetIoFilePropertyCallback',
    'GlobalDefReaderCallbacks_SetIoRegularFileCallback',
    'GlobalDefReaderCallbacks_SetIoDirectoryCallback',
    'GlobalDefReaderCallbacks_SetIoHandleCallback',
    'GlobalDefReaderCallbacks_SetIoPreCreatedHandleStateCallback',
    'GlobalDefReaderCallbacks_SetCallpathParameterCallback',
    'GlobalDefReaderCallbacks_SetInterCommCallback',
]




# Generated by OTF2 Template Engine

import _otf2
from . import events
from .attribute_list import AttributeList
from .definitions import Location


class BufferedEventReader(object):
    def __init__(self, reader, batch_events):
        self.reader = reader
        self.batch_events = batch_events
        self._selected_locations = None
        self._handle = None

    def _append(self, event_type, location_ref, time, _, attribute_list, *args):
        event = event_type._construct(self.reader.definitions, time, *args)
        event.attributes = AttributeList._construct(self.reader.definitions, attribute_list)
        location = self.reader.definitions.locations[location_ref]
        t = (location, event)
        self.buffer.append(t)

    def __call__(self, locations):
        if self._selected_locations is not None:
            raise RuntimeError("Cannot specify locations for an event reader twice.")
        if isinstance(locations, Location):
            locations = locations,

        self._selected_locations = locations

        return self

    def __iter__(self):
        if self._handle is not None:
            raise EOFError("Cannot iterate over trace twice.")
        self._handle = self.reader._get_global_evt_reader_handle(self._selected_locations)

        reader_callbacks = _otf2.GlobalEvtReaderCallbacks_New()
        self._set_global_event_reader_callbacks(reader_callbacks)
        _otf2.GlobalEvtReader_SetCallbacks(self._handle, reader_callbacks, None)
        _otf2.GlobalEvtReaderCallbacks_Delete(reader_callbacks)

        while True:
            self.buffer = []
            read_events = _otf2.GlobalEvtReader_ReadEvents(self._handle, self.batch_events)
            for loc_event in self.buffer:
                yield loc_event

            if read_events < self.batch_events:
                break

    def __len__(self):
        locations = self._selected_locations
        if locations is None:
            locations = self.reader._locations
        return sum(location.number_of_events for location in locations)

    def _set_global_event_reader_callbacks(self, cbs):
@otf2 for event in events:
        _otf2.GlobalEvtReaderCallbacks_Set@@event.name@@Callback(cbs, self._@@event.lower@@)
@otf2 endfor
@otf2 for event in events:

    def _@@event.lower@@(self, *args):
        self._append(events.@@event.name@@, *args)
@otf2 endfor



#!/usr/bin/env python
"""
This unit test checks the meta/base class infrastructure for definition classes
This is NOT an instruction how to use the otf2 module.
"""
import unittest

from otf2.definitions import _NamedRefDefinition, no_default, String
from otf2.registry import _RefRegistry
from otf2.enums import Type


class Unicorn(_NamedRefDefinition):
    _fields = (('horn', no_default, String),
               ('description', ''),
               ('legs', 4),
               ('sex', no_default, String),
               ('type', Type.FLOAT))
    _ref_name = 'metric'


class MockRegistry(object):
    def _update(self, old_key, unicorn):
        self.last_updated = unicorn


class TestDefinitions(unittest.TestCase):
    def test_valid(self):
        registry = MockRegistry()
        corn_silky_pony = Unicorn(registry, 42, 'Corn Silky Pony', '>>>>>', sex='m')
        self.assertEqual(corn_silky_pony._ref, 42)
        self.assertEqual(corn_silky_pony.name, 'Corn Silky Pony')
        self.assertEqual(corn_silky_pony.horn, '>>>>>')
        self.assertEqual(corn_silky_pony.description, '')
        self.assertEqual(corn_silky_pony.legs, 4)
        self.assertEqual(corn_silky_pony.sex, 'm')
        self.assertEqual(corn_silky_pony.type, Type.FLOAT)

        ivy_pink_ears = Unicorn(registry, 0, 'Ivy Pink Eears', horn='>>>', sex='f')
        self.assertEqual(ivy_pink_ears.horn, '>>>')
        self.assertEqual(ivy_pink_ears.sex, 'f')

        marigold_dapple_damsel = Unicorn(registry, 1, sex='f', horn='<<<<')
        self.assertEqual(marigold_dapple_damsel.name, 'Unicorn #1')
        self.assertEqual(marigold_dapple_damsel.sex, 'f')
        self.assertEqual(marigold_dapple_damsel.horn, '<<<<')

        chestnut_celestial_cheeks = Unicorn(registry, 0, 'Chestnut Celestial Cheeks', '>>>',
                                            'sparkly', type=Type.INT8, legs=6, sex='m')
        self.assertEqual(chestnut_celestial_cheeks.horn, '>>>')
        self.assertEqual(chestnut_celestial_cheeks.description, 'sparkly')
        self.assertEqual(chestnut_celestial_cheeks.type, Type.INT8)
        self.assertEqual(chestnut_celestial_cheeks.legs, 6)
        self.assertEqual(chestnut_celestial_cheeks.sex, 'm')

        robin_velvet_foal = Unicorn(registry, 0, 'Robin Velvet Foal',  '****', 'shiny', 4, 'm',
                                    Type.FLOAT)
        self.assertEqual(robin_velvet_foal.name, 'Robin Velvet Foal')
        self.assertEqual(robin_velvet_foal.horn, '****')
        self.assertEqual(robin_velvet_foal.description, 'shiny')
        self.assertEqual(robin_velvet_foal.legs, 4)
        self.assertEqual(robin_velvet_foal.sex, 'm')
        self.assertEqual(robin_velvet_foal.type, Type.FLOAT)

        # A tragic accident :-(
        robin_velvet_foal.legs = 3
        self.assertEqual(robin_velvet_foal.legs, 3)
        self.assertEqual(registry.last_updated, robin_velvet_foal)

    def test_invalid(self):
        registry = MockRegistry()
        with self.assertRaises(TypeError):
            # Many missing
            Unicorn(registry, 0)

        with self.assertRaises(TypeError):
            # Missing horn
            Unicorn(registry, 0, 'Incomplete Indigo Ingo', sex='m')

        with self.assertRaises(TypeError):
            # Wrong type
            Unicorn(registry, 0, 'Thyme Beautiful Stallion', '>>>>>', sex=1)

        with self.assertRaises(TypeError):
            # Unused keyword argument
            Unicorn(registry, 0, horn='^', sex='f', extra=42)

        with self.assertRaises(TypeError):
            # Duplicate positional/keyword argument
            Unicorn(registry, 0, 'Parsley Velvet Trail', '^', sex='f', horn='>>')

        with self.assertRaises(TypeError):
            # Too many positional arguments
            Unicorn(registry, 0, 'Sage Beautiful Legs',  '****', 'shiny', 6, 'm', Type.FLOAT, 6)

        with self.assertRaises(TypeError):
            # Two horns not allowed
            Unicorn(registry, 0, "Parsley Jolly Legs", (">>>", ">>>>"), sex='m')

        corn_silky_pony = Unicorn(registry, 42, 'Corn Silky Pony', '>>>>>', sex='m')
        with self.assertRaises(AttributeError):
            corn_silky_pony.evil = True

        with self.assertRaises(TypeError):
            corn_silky_pony.legs = "4"

if __name__ == '__main__':
    unittest.main()



import os
import os.path
import unittest
from shutil import rmtree
from tempfile import mkdtemp
import time

import otf2

from otf2_writer import generate_trace

TIMER_GRANULARITY = 1000000


def t():
    return int(round(time.time() * TIMER_GRANULARITY))


class TestOTF2UserData(unittest.TestCase):

    archive_name = "otf2_test_trace"

    def setUp(self):
        self.old_cwd = os.getcwd()
        self.tmp_dirname = mkdtemp(prefix=os.path.basename(os.path.abspath(__file__))[:-3] + '_tmp', dir=self.old_cwd)
        os.chdir(self.tmp_dirname)

    def tearDown(self):
        os.chdir(self.old_cwd)
        if os.getenv('KEEP_TEST_OUTPUT', '') != '':
            print(self.tmp_dirname)
        else:
            rmtree(self.tmp_dirname)

    def read_trace(self, archive_name):
        with otf2.reader.open(archive_name) as trace:
            for location in trace.definitions.locations:
                self.assertEqual(location.number_of_events, 4)

    def test_integrity(self):
        self.definitions = generate_trace(self.tmp_dirname)
        self.read_trace(os.path.join(self.tmp_dirname, "traces.otf2"))


if __name__ == '__main__':
    unittest.main()




''' Generated by OTF2 Template Engine '''

import ctypes

from .Config import EnumBase, EnumBitset

@otf2 for enum in enums|def_enums:
@otf2  if enum is bitset_enum:
class @@enum.type.py_ctype@@(EnumBitset, @@enum.type.py_underlying_ctype@@):
@otf2  else:
class @@enum.type.py_ctype@@(EnumBase, @@enum.type.py_underlying_ctype@@):
@otf2  endif
    pass

@otf2  for entry in enum.entries|py_def_enum_entries:
@@entry.name[5:]@@ = @@enum.type.py_ctype@@._construct(@@entry.value@@, "@@entry.suffix@@")
@otf2  for alias in entry.aliases:
@@alias.name[5:]@@ = @@enum.type.py_ctype@@._construct(@@entry.value@@, "@@alias.suffix@@")
@otf2  endfor
@otf2  endfor
@otf2  if enum is enum_with_terminator:
@@enum.terminator.name[5:]@@ = @@enum.type.py_ctype@@(@@enum.terminator.value@@)
@otf2  endif

@otf2 endfor
METRIC_ACCUMULATED_START = MetricMode._construct(METRIC_VALUE_ACCUMULATED.value | METRIC_TIMING_START.value, "ACCUMULATED_START")
METRIC_ACCUMULATED_POINT = MetricMode._construct(METRIC_VALUE_ACCUMULATED.value | METRIC_TIMING_POINT.value, "ACCUMULATED_POINT")
METRIC_ACCUMULATED_LAST  = MetricMode._construct(METRIC_VALUE_ACCUMULATED.value | METRIC_TIMING_LAST.value, "ACCUMULATED_LAST")
METRIC_ACCUMULATED_NEXT  = MetricMode._construct(METRIC_VALUE_ACCUMULATED.value | METRIC_TIMING_NEXT.value, "ACCUMULATED_NEXT")
METRIC_ABSOLUTE_POINT    = MetricMode._construct(METRIC_VALUE_ABSOLUTE.value    | METRIC_TIMING_POINT.value, "ABSOLUTE_POINT")
METRIC_ABSOLUTE_LAST     = MetricMode._construct(METRIC_VALUE_ABSOLUTE.value    | METRIC_TIMING_LAST.value, "ABSOLUTE_LAST")
METRIC_ABSOLUTE_NEXT     = MetricMode._construct(METRIC_VALUE_ABSOLUTE.value    | METRIC_TIMING_NEXT.value, "ABSOLUTE_NEXT")
METRIC_RELATIVE_POINT    = MetricMode._construct(METRIC_VALUE_RELATIVE.value    | METRIC_TIMING_POINT.value, "RELATIVE_POINT")
METRIC_RELATIVE_LAST     = MetricMode._construct(METRIC_VALUE_RELATIVE.value    | METRIC_TIMING_LAST.value, "RELATIVE_LAST")
METRIC_RELATIVE_NEXT     = MetricMode._construct(METRIC_VALUE_RELATIVE.value    | METRIC_TIMING_NEXT.value, "RELATIVE_NEXT")

__all__ = [
    @otf2 for enum in enums|def_enums:
    '@@enum.type.py_ctype@@',
    @otf2 for entry in enum.entries:
    '@@entry.name[5:]@@',
    @otf2 endfor
    @otf2 if enum is enum_with_terminator:
    '@@enum.terminator.name[5:]@@',
    @otf2 endif
    @otf2 endfor
]
